/**
 * Le controleur pour la representation graphique de l'architecture
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 * @author Yves Lhuillier
 */

package Controlers;

import Interface.IArchi.UIArchi;
import Model.MArchi;
import Utilitaires.PixConstants;
import java.awt.event.*;


public class ArchiControl implements ActionListener
{
  /** L'image de l'interface */
  private UIArchi mArchiInterface;
  /** Le modele de l'architecture */
  private MArchi mArchiModel;

  /**
   * Le constructeur
   * @param inArchiModel : modele d'architecture
   * @param inArchiInterface : interface d'architecture
   */
  public ArchiControl( MArchi inArchiModel, UIArchi inArchiInterface )
  {
    mArchiModel = inArchiModel;
    mArchiInterface = inArchiInterface;
  }

  /**
   * Methode de l'interface ActionListener
   * @param e : l'evnement capte
   * @see java.awt.event.ActionListener
   * @see java.awt.event.ActionEvent
   */
  public void actionPerformed( ActionEvent inUnused ) { mArchiInterface.animate(); }

  /** preaffichage des images */
  public void showAllpics() { mArchiInterface.getArchi().showAllpics(); }

  /** Nettoie le schema */
  public void clear() { mArchiInterface.getArchi().repaint(); }

  /** Stoppe l'animation en cours */
  public void stopAnim() { mArchiInterface.stopAnim(); }

  /** methode qui ajoute une animation a la liste */
  public void addAnimState( int anim ) { mArchiInterface.addAnimState( anim ); }

  /** Methode qui demarre les animations */
  public void startAnim() { mArchiInterface.startAnim(); }

  /** Methode qui poursuit l'execution lorsqu'une animation est finie */
  public void continueExec() { mArchiModel.continueExec(); }

  /** Met a 0 le n d'animation */
  public void resetColor() { mArchiInterface.resetColor(); }

  /** La methode qui dit si la lecture d'une instruction doit tre animee ou non */
  public void setAnimLect(boolean b) { mArchiModel.setAnimLect( b ); }

  /**
   * Met a jour le texte dans l'architecture
   * @ param inStr : une chaine de carctres representant le message a afficher
   */
  public void setMsg( String inStr ) { mArchiInterface.setMsg( inStr ); }

  /**
   * Met a jour le compteur de programmes dans l'architecture
   * @ param inPC : une entier donnant la valeur du compteur de programmes
   */
  public void updatePC( int inPC ) { mArchiInterface.getArchi().setDrawingPC( inPC ); }

  /**
   * Met a jour l'operation de l'ual
   * @param inStr : la chaine de caractere representant l'operation realise
   */
  public void setUalInfo( String inStr ) { mArchiInterface.getArchi().setUal( inStr ); }

  /**
   * Met a jour le registre1 dans l'image
   * @param inStr : la valeur a mettre a jour
   */
  public void setReg1Info( String inStr ) { mArchiInterface.getArchi().setRegA( inStr ); }

  /**
   * Met a jour le registre2 dans l'image
   * @param inStr : la valeur a mettre a jour
   */
  public void setReg2Info( String inStr ) { mArchiInterface.getArchi().setRegB( inStr ); }

  /**
   * Met a jour le registre3 dans l'image
   * @param inStr : la valeur a mettre a jour
   */
  public void setReg3Info( String inStr ) { mArchiInterface.getArchi().setRegC( inStr ); }

  /** Met a jour les affichages dans l'architecture */
  public void updateArchi() {  mArchiInterface.getArchi().updateArchi(); }

  /**
   * Accs au message dans l'archi
   * @return Le message
   */
  public String getMsg() { return mArchiInterface.getMsg().getText(); }
}
