/**
 * Le controleur pour la barre de commandes
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Controlers;

import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.util.Vector;

import Interface.ICmd.UICmd;
import Model.Engine;

public class CmdControl implements ActionListener
{
  /** Le paneau contenant les boutons de commandes a gerer */
  private UICmd   mCmdInterface;
  /** Le modele de commande */
  private Engine  mCmdModel;

  /** Le constructeur
   * @param inCmdInterface:  le panel de commande
   * @param inCmdModel:      le modele de commandes
   */
  public CmdControl( Engine inCmdModel, UICmd inCmdInterface )
  {
    mCmdInterface = inCmdInterface;
    mCmdModel = inCmdModel;
  }

  /** la methode d'acces au mCmdModel */
  public Engine getModel() { return mCmdModel; }

  /** Rend le popupMenu non vivible */
  public void setPopupNotVisible() { mCmdInterface.getTimerPopup().setVisible( false ); }

  /** La methode qui construit un gestionnare d'evenements */
  public void createListener()
  {
    mCmdInterface.getPlay().addActionListener( this );
    mCmdInterface.getPause().addActionListener( this );
    mCmdInterface.getStop().addActionListener( this );
    mCmdInterface.getBack().addActionListener( this );
    mCmdInterface.getNext().addActionListener( this );
    mCmdInterface.getBackStep().addActionListener( this );
    mCmdInterface.getNextStep().addActionListener( this );
    mCmdInterface.getTimer().addActionListener( this );
    Vector items = mCmdInterface.getItemVector();
    for( int j=0; j < items.size(); j++ )
      ((JMenuItem)items.elementAt( j )).addActionListener( this );
  }

  /** Methode qui sert a forcer la selection du bouton play */
  public void setPlaySelected() { mCmdInterface.getPlay().setSelected( true ); }

  /** Methode qui sert a forcer la selection du bouton pause */
  public void setPauseSelected() { mCmdInterface.getPause().setSelected( true ); }

  /** Methode qui sert a forcer la selection du bouton stop */
  public void setStopSelected() { mCmdInterface.getStop().setSelected( true ); }

  /**
   * Methode qui sert a (des)activer les boutons de lectures
   * @param inActive: L'etat d'activation
   */
  public void setStepsEnabled( boolean inActive )
  {
    mCmdInterface.getNext().setEnabled( inActive );
    mCmdInterface.getBack().setEnabled( inActive );
    mCmdInterface.getNextStep().setEnabled( inActive );
    mCmdInterface.getBackStep().setEnabled( inActive );
  }

  /**
   * La methode qui est appelee lorsqu'un evenement (on suppose un clic) se produit sur l'un des boutons.
   * @param inEvent: l'evenement qui s'est produit
   */
  public void actionPerformed( ActionEvent inEvent )
  {
    Object source = inEvent.getSource();
    
    if( source == mCmdInterface.getPlay() )     { mCmdModel.play(); }     else
    if( source == mCmdInterface.getPause() )    { mCmdModel.pause(); }    else
    if( source == mCmdInterface.getStop() )     { mCmdModel.stop(); }     else
    if( source == mCmdInterface.getBack() )     { mCmdModel.back(); }     else
    if( source == mCmdInterface.getNext() )     { mCmdModel.next(); }     else
    if( source == mCmdInterface.getBackStep() ) { mCmdModel.backStep(); } else
    if( source == mCmdInterface.getNextStep() ) { mCmdModel.nextStep(); } else
    if( source == mCmdInterface.getTimer() )
    {
      JPopupMenu lPopupMenu = mCmdInterface.getTimerPopup();
      lPopupMenu.setVisible( true );
      lPopupMenu.show( mCmdInterface.getTimer(), 0, -mCmdInterface.getTimerPopup().getHeight() );
    }
    else
    {
      Integer time = Integer.valueOf( ((JMenuItem)source).getText() );
      mCmdModel.setTime( time.intValue() );
      mCmdInterface.getTimer().setText( "" + time.intValue() );
    }
  }
}
