/**
 * Le controleur pour la fenetre d'edition
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Controlers;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFileChooser;
import java.util.Vector;
import java.io.*;

import Interface.IEdit.UIEdit;
import Interface.IEdit.EditExt;
import Model.MEdit;

public class EditControl implements ActionListener
{
  /** La fenetre de l'editeur */
  private UIEdit  mEditInterface;
  /** Le modele de l'editeur */
  private MEdit   mEditModel;

  /**
   * Le constructeur
   * @param em : modele de l'editeur
   * @param ue : interface de l'editeur
   */
  public EditControl( MEdit inEditModel, UIEdit inEditInterface )
  {
    mEditInterface = inEditInterface;
    mEditModel = inEditModel;
  }

  /** La methode qui cree un gestionnaire d'evenements pour les boutons de la fenetre d'edition */
  public void createListener()
  {
    mEditInterface.getLoadBtn().addActionListener( this );
    mEditInterface.getSaveBtn().addActionListener( this );
    mEditInterface.getEditBtn().addActionListener( this );
  }

  /** La methode qui cree un gestionnaire d'evenements pour les boutons de la fenetre d'edition EXTERNE */
  public void createListenerForEditor()
  {
    EditExt lEditExt = mEditInterface.getEditExt();
    lEditExt.getLoadBtn().addActionListener( this );
    lEditExt.getSaveBtn().addActionListener( this );
    lEditExt.getOk().addActionListener( this );
    lEditExt.getCancel().addActionListener( this );
  }

  /** La methode qui cree un gestionnaire d'evenements pour la fenetre d'erreur */
  public void createListenerForError() { mEditInterface.getErrorOk().addActionListener( this ); }

  /**
   * La methode qui lance l'analyse lexicale et syntaxique
   * @exception Exception
   */
  public Vector analyse() throws Exception
  {
    mEditInterface.unEdit();
    return mEditModel.analyse();
  }

  /**
   * La methode qui montre la boite d'erreur
   * @param inMsg : le message a afficher
   */
  public void showErrorSyntax( String inMsg ) { mEditInterface.showErrorSyntax( inMsg ); }

  /**
   * La methode qui montre la boite d'erreur
   * @param inMsg : le message a afficher
   */
  public void showErrorFormat( String inMsg ) { mEditInterface.showErrorFormat( inMsg ); }

  /**
   * La methode qui montre la boite d'erreur pour une erreur de saut
   * @param inMsg : le message a afficher
   */
  public void showErrorJump( String inMsg ) { mEditInterface.showErrorJump( inMsg ); }

  /**
   * La methode qui remplit la fenetre de l'editeur avec un nouveau contenu.
   * @param inContent : un tableau contenant une suite d'instructions assembleur
   * @see Interface.IEdit.UIEdit#setContent( Vector v )
   */
  public void setUIEdit( Vector inContent ) { mEditInterface.setContent( inContent ); }

  /**
   * La methode qui remplit le modele de l'editeur avec un nouveau contenu.
   * @param content : un tableau contenant une suite d'instructions assembleur
   */
  public void setMEdit( Vector content ) { mEditModel.setContent( content ); }

  /**
   * La methode qui change le contenu d'une case du tableau d'instructions assemebleur dans le modele.
   * @param valeur : l'instruction a charger
   * @param valeur : la position ou l'ajouter
   */
  public void setMEditAt( String valeur, int pos ) { mEditModel.setContentAt( valeur, pos ); }

  /** Stoppe l'edition du programme */
  public void unEdit() { mEditInterface.unEdit(); }

  /**
   * Rend l'editeur accessible ou non
   * @param inAccess : accessibilite
   */
  public void setEditEnabled( boolean inAccess ) { mEditInterface.setEditable( inAccess ); }

  /**
   * Methode qui permet de selectionner une ligne dans le tableau
   * @param lIndex: l'index de la ligne
   */
  public void setSelected( int lIndex ) { mEditInterface.setSelected( lIndex ); }

  /**
   * La methode qui supprime une ligne d'instruction
   * @param inPos : l'instruction a supprimer
   */
  public void removeLine( int inPos ) { mEditModel.removeLine( inPos ); }

  /**
   * La methode d'acces au moteur d'edition
   */
  public MEdit getModel() { return mEditModel; }

  /**
   * Methode servant a trouver le numero de ligne d'une instruction erronee
   * @param l : le numero de la ligne sans tenir compte des espaces dans l'editeur
   * @return le numero de ligne en tenant compte des espaces dans l'editeur
   */
  public int seekPosInEdit( int l ) { return mEditInterface.seekPos( l ); }

  /**
   * La methode qui est appelee lorsqu'un evenement se produit sur un bouton de la fenetre editeur.
   * @param inEvent : l'evenement qui s'est produit
   */
  public void actionPerformed( ActionEvent inEvent )
  {
    Object source=inEvent.getSource();
    JFileChooser lFileChoice;
    
    if( source == mEditInterface.getLoadBtn() )
    {
      lFileChoice = mEditInterface.getFileChoice();
      lFileChoice.setSelectedFile( null );
      lFileChoice.rescanCurrentDirectory();
      int returnVal = lFileChoice.showOpenDialog(mEditInterface );
      File lFile = lFileChoice.getSelectedFile();
      if( returnVal == JFileChooser.APPROVE_OPTION && lFile != null )
      {
        try{ mEditModel.setContent( new FileReader( lFile ) ); }
        catch ( FileNotFoundException er ) {}
      }
    }
    else if( source == mEditInterface.getSaveBtn() )
    {
      mEditInterface.unEdit();
      lFileChoice = mEditInterface.getFileChoice();
      lFileChoice.setSelectedFile(null );
      lFileChoice.rescanCurrentDirectory();
      int returnVal = lFileChoice.showSaveDialog(mEditInterface );
      File lFile = lFileChoice.getSelectedFile();
      if( returnVal == JFileChooser.APPROVE_OPTION && lFile != null )
        mEditModel.sauver( lFile );
    }
    else if( source == mEditInterface.getErrorOk() )
    {
      mEditInterface.getTable().revalidate();
      mEditInterface.getTable().repaint();
      mEditInterface.getError().setModal( false );
      mEditInterface.getError().setVisible( false );
      mEditInterface.getError().dispose();
    }
    else if( source==mEditInterface.getEditBtn() )
    {
      mEditInterface.unEdit();
      mEditInterface.getTopLevelAncestor().setEnabled( false );
      mEditInterface.showEditor(mEditModel.getText() );
    }
    if( mEditInterface.getEditExt()!=null )
    {
      if( source == mEditInterface.getEditExt().getLoadBtn() )
      {
        lFileChoice = mEditInterface.getFileChoice();
        lFileChoice.setSelectedFile( null );
        lFileChoice.rescanCurrentDirectory();
        int returnVal = lFileChoice.showOpenDialog( mEditInterface );
        File lFile = lFileChoice.getSelectedFile();
        if(returnVal == JFileChooser.APPROVE_OPTION && lFile != null )
        {
          try{ mEditInterface.getEditExt().charger( new FileReader(lFile) );}
          catch (FileNotFoundException er) {}
        }
      }
      if( source == mEditInterface.getEditExt().getSaveBtn() )
      {
        lFileChoice = mEditInterface.getFileChoice();
        lFileChoice.setSelectedFile(null );
        lFileChoice.rescanCurrentDirectory();
        int returnVal = lFileChoice.showSaveDialog(mEditInterface );
        File lFile = lFileChoice.getSelectedFile();
        if( returnVal == JFileChooser.APPROVE_OPTION && lFile != null )
          mEditInterface.getEditExt().sauver( lFile );
      }
      if( source == mEditInterface.getEditExt().getOk() )
      {
        String txt = mEditInterface.getEditExt().getText();
        mEditInterface.getEditExt().dispose();
        mEditInterface.getTopLevelAncestor().setEnabled( true );
        mEditModel.setContentFromString(txt );
      }
      if( source == mEditInterface.getEditExt().getCancel() )
      {
        mEditInterface.getEditExt().dispose();
        mEditInterface.getTopLevelAncestor().setEnabled( true );
      }
    }
  }
}
