/**
 * Le controleur pour la RAM
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Controlers;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.util.Vector;

import Utilitaires.*;
import Interface.IRam.UIRam;
import Controlers.RegControl;
import Model.MRam;

public class RamControl implements ItemListener , ActionListener
{
  /** La fenetre de la RAM */
  private UIRam       mRamInterface;
  /** Le modele de la RAM */
  private MRam        mRamModel;
  /** Le controleur des regs */
  private RegControl  mRegControl;
  /** Le precedent choix d'affichage de la RAM */
  private String      mRamDisplay = "Tout";
  /** Le precedent choix du type du contenu de la RAM */
  private String      mRamContent = "Binaire";

  /**
   * Le constructeur
   * @param inRamControl : modele de la RAM
   * @param inRegControl : modele du banc de registre
   * @param inRamInterface : interface de la RAM
   */
  public RamControl( MRam inRamControl, UIRam inRamInterface, RegControl inRegControl )
  {
    mRamModel   = inRamControl;
    mRamInterface = inRamInterface;
    mRegControl   = inRegControl;
  }

  /** La methode qui cree un gestionnaire d'evenements pour les combobox de la fenetre de RAM */
  public void createListener()
  {
    mRamInterface.getComboContenu().addItemListener( this );
    mRamInterface.getComboRam().addItemListener( this );
  }

  /** Creation d'un gestionnaire d'evenements pour la fenetre de choix de la RAM */
  public void createListenerForChoice()
  {
    mRamInterface.getRamChoiceFrame().getDone().addActionListener( this );
    mRamInterface.getRamChoiceFrame().getCancel().addActionListener( this );
  }

  /** Methode d'acces au type de la ram */
  public String getRamType() { return mRamDisplay; }

  /** La methode qui rafraichit la Ram */
  public void refreshReg()   { mRamInterface.setAll(); }

  /**
   * La methode qui remplit la fenetre de la RAM avec le contenu du modele de RAM.
   * @see Interface.IRam.UIRam#setRam( Vector )
   * @see Model.MRam#getMem()
   */
  public void setRamInUI() { mRamInterface.setRam( mRamModel.getMem() ); }

  /**
   * La methode qui remplit le modele de la RAM a partir d'un changement dans l'interface.
   * @param inVal : la nouvelle valeur a mettre dans le modele de RAM
   * @param inAddr : l'adresse ou le contenu prend la valeur inVal
   */
  public void setRamFromUI( Integer inVal, int inAddr ) { mRamModel.setMemAt( inVal, inAddr ); }

  /**
   * La methode qui remplit le modele de la RAM a partir d'un changement
   * dans le modele lors de l'assemblage
   * @param inBin : la nouvelle valeur a mettre dans le modele et l'interface de RAM (en binaire)
   * @param inAddr : l'adresse ou le contenu prend la valeur lValue
   */
  public void setRamFromModel( String inBin, int inAddr )
  {
    Integer lValue = new Integer ( Utils.signedBinToInt( inBin ) );
    mRamModel.setMemAt( lValue, inAddr );
    mRamInterface.setMemAt( lValue, inAddr );
    if ( mRamDisplay.equals("Modifi") )
      mRamInterface.setAlter( mRamModel.getAlteredStorage(), mRamModel.getAlteredAddr() );
    if ( mRamDisplay.equals("Au choix"))
      mRamInterface.setChoiceAt( lValue, new Integer( inAddr ) );
  }

  /**
   * La methode qui change le type d'affichage du contenu en binaire
   * @see Interface.IRam.UIRam#setAffichage( int )
   */
  public void setToBinary() { mRamInterface.setAffichage( PixConstants.BINAIRE ); }

  /**
   * La methode qui change le type d'affichage du contenu en hexadecimal
   * @see Interface.IRam.UIRam#setAffichage( int )
   */
  public void setToHexadecimal() { mRamInterface.setAffichage( PixConstants.HEXADECIMAL ); }

  /**
   * La methode qui change le type d'affichage du contenu en decimal
   * @see Interface.IRam.UIRam#setAffichage( int )
   */
  public void setToDecimal() { mRamInterface.setAffichage( PixConstants.DECIMAL ); }

  /**
   * La methode qui initialise l'affichage par modification
   * @see Interface.IRam.UIRam#setAffichage(int)
   */
  public void initAlter() { mRamModel.newAlter(); }

  /**
   * La methode qui est appelee lorsqu'un evenement se produit sur un combobox de la fenetre de RAM.
   * @param evt : l'evenement qui s'est produit
   */
  public void itemStateChanged( ItemEvent evt )
  {
    Object source  = evt.getSource();
    Object changed = evt.getItem();

    if( source == mRamInterface.getComboRam() )
    {
      if ( changed.equals("Tout") && !mRamDisplay.equals("Tout") )
      {
        mRamDisplay = "Tout";
        mRamInterface.setAll();
      }
      if ( changed.equals("Modifi") && !mRamDisplay.equals("Modifi") )
      {
        mRamDisplay = "Modifi";
        mRamInterface.setAlter( mRamModel.getAlteredStorage(), mRamModel.getAlteredAddr() );
      }
      if ( changed.equals("Au choix") && !mRamDisplay.equals("Au choix") )
      {
        mRamDisplay = "Au choix";
        mRamInterface.getTopLevelAncestor().setEnabled(false );
        mRamInterface.showChoice();
      }
    }
    if( source == mRamInterface.getComboContenu() )
    {
      if ( changed.equals("Binaire") && !mRamContent.equals("Binaire") )
      {
        mRamContent="Binaire";
        setToBinary();
        mRegControl.setToBinary();
      }
      if ( changed.equals("Dcimal") && !mRamContent.equals("Dcimal") )
      {
        mRamContent="Dcimal";
        setToDecimal();
        mRegControl.setToDecimal();
      }
      if ( changed.equals("Hexadcimal") && !mRamContent.equals("Hexadcimal") )
      {
        mRamContent="Hexadcimal";
        setToHexadecimal();
        mRegControl.setToHexadecimal();
      }
    }
  }


  /**
   * La mthode qui est appele lorsqu'un evenement se produit
   * sur un bouton de la fenetre de choix de la RAM:RamChoiceFrame
   * @param evt : l'vnement qui s'est produit
   */

  public void actionPerformed(ActionEvent evt)
  {
    Object source=evt.getSource();

    if( source == mRamInterface.getRamChoiceFrame().getCancel() )
    {
      mRamInterface.getComboRam().setSelectedItem("Tout" );
      mRamInterface.getRamChoiceFrame().dispose();
      mRamInterface.setNewRam( mRamInterface.getMem(), mRamInterface.getMemAddr() );
      mRamInterface.getTopLevelAncestor().setEnabled( true );
      mRamContent="Binaire";
      mRamDisplay="Tout";
    }
    if( source == mRamInterface.getRamChoiceFrame().getDone() )
    {
      mRamInterface.getRamChoiceFrame().dispose();
      // Remplit un vecteur dans UIRam
      int[] selected = mRamInterface.getRamChoiceFrame().getListeCases().getSelectedIndices();
      Vector newRam = new Vector();
      Vector newCases = new Vector();
      Integer lInt;
      for( int i=0; i < java.lang.reflect.Array.getLength(selected ); i++ )
      {
        lInt = new Integer( (java.lang.reflect.Array.get( selected, i )).toString() );
        newCases.addElement( lInt );
        newRam.addElement( (Integer)mRamInterface.getMem().elementAt( lInt.intValue()) );
      }
      mRamInterface.setChoice( newRam, newCases );
      mRamInterface.getTopLevelAncestor().setEnabled( true );
    }
  }
}
