/**
 * Le controleur pour la RAM
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 * @author Yves Lhuillier
 */

package Controlers;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.util.Vector;

import Utilitaires.*;
import Interface.IReg.UIReg;
import Model.MReg;

public class RegControl
{
  /** Le modele des registres */
  private MReg mRegModel;
  /** La fenetre des registres */
  private UIReg mRegInterface;


  /**
   * Le constructeur
   * @param inRegModel : modele des regs
   * @param inRegInterface : interface des regs
   */
  public RegControl( MReg inRegModel, UIReg inRegInterface )
  {
    mRegModel = inRegModel;
    mRegInterface = inRegInterface;
  }


  /** La methode qui rafraichit les registres */
  public void refreshReg() { mRegInterface.refreshReg(); }

  /**
   * La methode qui remplit la fenetre des registres avec le contenu du modele des registres.
   * @see Interface.IReg.UIReg#setReg
   * @see Model.MReg#getReg
   */
  public void setRegInUI() { mRegInterface.setReg( mRegModel.getReg() ); }

  /**
   * La methode qui remplit le registre RCC
   * @see Interface.IReg.UIReg#setRcc
   * @see Model.MReg#getRcc
   */
  public void setRccInUI() { mRegInterface.setRcc( mRegModel.getRcc() ); }

  /**
   * La methode qui met a jour un registre dans l'interface
   * @param v : la valeur
   * @param col : le n de registre
   *
   */
  public void setRegInUIAt( Integer v, int col ) { mRegInterface.setRegAt( v, col ); }

  /**
   * La methode qui met a jour un flag du registre RCC
   * @param v : la valeur
   * @param i : le n de flag
   *
   */
  public void setRccInUIAt( int inValue, int inIndex )
  {
    switch( inIndex )
    {
    case PixConstants.RCCC: mRegInterface.setRccC( inValue ); break;
    case PixConstants.RCCZ: mRegInterface.setRccZ( inValue ); break;
    case PixConstants.RCCN: mRegInterface.setRccN( inValue ); break;
    case PixConstants.RCCV: mRegInterface.setRccV( inValue ); break;
    };
  }

  /**
   * La methode qui remplit le modele des registres a partir d'un changement dans l'interface.
   * @param val : la nouvelle valeur a mettre dans le modele de registres
   * @param col : le numero de regitre
   */
  public void setRegFromUI( Integer inValue, int lIndex ) { mRegModel.setRegAt( inValue, lIndex ); }

  /**
   * La methode qui change le type d'affichage du contenu en binaire
   * @see Interface.IReg.UIReg#setAffichage(int)
   */
  public void setToBinary() { mRegInterface.setAffichage( PixConstants.BINAIRE ); }

  /**
   * La methode qui change le type d'affichage du contenu en hexadecimal
   * @see Interface.IReg.UIReg#setAffichage(int)
   */
  public void setToHexadecimal() { mRegInterface.setAffichage( PixConstants.HEXADECIMAL ); }

  /**
   * La methode qui change le type d'affichage du contenu en decimal
   * @see Interface.IReg.UIReg#setAffichage(int)
   */
  public void setToDecimal() { mRegInterface.setAffichage( PixConstants.DECIMAL ); }
}
