/**
 * Le controleur de la fenetre principale UI
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Controlers;

import java.awt.event.*;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import java.io.*;

import Interface.UI.*;
import Model.MEdit;
import Utilitaires.*;


public class UIControl implements ActionListener
{
  /** Le menu deroulant */
  private static Menu sMenu;
  /** Le moteur d'edition pour charger et sauver des fichiers */
  private static MEdit sEditEngine;
  /** La fenetre principale */
  private static UI sInterface;
  /** La dialog Box du mode demo (tutorial) */
  private Tutorial mTuto;
  /** La fentre d'aide de depart */
  private HelpView mHelp;
  /** La fentre "A propos" */
  private About mAbout = new About();

  /**
   * Le constructeur
   * @param inEditEngine : moteur de l'editeur
   * @param inInterface :  interface de la RAM
   */
  public UIControl( MEdit inEditEngine, UI inInterface )
  {
    sMenu = inInterface.getMenu();
    sEditEngine = inEditEngine;
    sInterface = inInterface;
  }

  /**
   * Rend les menu Fichier->Charger et Fichier->Sauver accessible ou non
   * @param b : true s'il est accessible, false sinon
   */
  public void setMenusEnabled( boolean inEnabled ) {
    sMenu.getOuvrir().setEnabled( inEnabled );
    sMenu.getSauver().setEnabled( inEnabled );
  }

  /**
   * La methode qui est appelee lorsqu'un evenement se produit sur le menu
   * @param e : l'evenement qui s'est produit
   */
  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();

    if( source == sMenu.getOuvrir() )
    {
      sInterface.getUIEdit().getFileChoice().setSelectedFile(null );
      sInterface.getUIEdit().getFileChoice().rescanCurrentDirectory();
      int returnVal = sInterface.getUIEdit().getFileChoice().showOpenDialog( sInterface.getUIEdit() );

      File lFile = sInterface.getUIEdit().getFileChoice().getSelectedFile();
      if( returnVal == JFileChooser.APPROVE_OPTION && lFile!=null )
      {
        try { sEditEngine.setContent( new FileReader(lFile) ); }
        catch ( FileNotFoundException er ) {}
      }
    }
    if( source == sMenu.getSauver() )
    {
      sInterface.getUIEdit().unEdit();
      sInterface.getUIEdit().getFileChoice().setSelectedFile(null );
      sInterface.getUIEdit().getFileChoice().rescanCurrentDirectory();
      int returnVal = sInterface.getUIEdit().getFileChoice().showSaveDialog( sInterface.getUIEdit() );

      File lFile = sInterface.getUIEdit().getFileChoice().getSelectedFile();
      if( returnVal == JFileChooser.APPROVE_OPTION && lFile != null )
        sEditEngine.sauver(lFile );
    }
    if( source == sMenu.getQuitter() )   { System.exit(0 ); }
    if( source == sMenu.getR640x480() )  { sInterface.setResolution(640,480 ); }
    if( source == sMenu.getR800x600() )  { sInterface.setResolution(800,600 ); }
    if( source == sMenu.getR1024x768() ) { sInterface.setResolution(1024,768 ); }
    if( source == sMenu.getDemo() )      { mTuto = new Tutorial(sInterface,this ); }
    if( source == sMenu.getAbout() )     { mAbout.show(); }
    if( source == sMenu.getNovice() ) {
      sInterface.getUIRam().setForbidden();
      sInterface.getUIReg().setForbidden();
      sInterface.setTitle("PixArchi : Mode Novice" );
      sInterface.setAnimLect( true );
      sMenu.getAnim().setSelected( true );
      sMenu.getAnim().setEnabled( false );
    }
    
    if( source == sMenu.getExpert() ) {
      sInterface.getUIRam().setAllowed();
      sInterface.getUIReg().setAllowed();
      sInterface.setTitle("PixArchi : Mode Expert" );
      sMenu.getAnim().setEnabled( true );
    }
    
    if( source == sMenu.getAnim() )
      sInterface.setAnimLect( sMenu.getAnim().isSelected() );
    if( source == sMenu.getAide() ) {
      mHelp = new HelpView("file:../Docs/Help/sommaire.html" );
      mHelp.show();
    }
    
    if( mTuto != null ) {
      if(source==mTuto.getQuitter()) { mTuto.quitter(); }
      if(source==mTuto.getOk()) {
        mTuto.suite();
        mTuto.repaint();
      }
    }
  }
}
