/**
 * La partie de l'interface  gerant le schema d'architecture.
 * Elle contient le schema de l'artchitecture et un label definissant l'etat d'execution
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IArchi;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.applet.Applet;

import Utilitaires.GridUtils;
import Utilitaires.PixConstants;
import Controlers.ArchiControl;


public class UIArchi extends JPanel

{
  /**
   * Le magnifique schema d'architecture !
   */
  private ImageIcon i = new ImageIcon("../Images/archi/archi800-600.gif" );
  private JLabel imageLabel = new JLabel() {
    public void paint(Graphics g) {
      super.paint(g );
      if (image!=null) image.paintImage(g );
    }
  };
  
  /**
   * Le controleur d'architecture
   */
  private ArchiControl control;
  /**
   * Le label contenant l'etat d'execution
   */
  private JLabel isEtat = new JLabel( "" );

  private Visual image  = new Visual( this );

  private JPanel texte  = new JPanel( new FlowLayout( FlowLayout.LEFT, 2, 0) );

  private JLabel etat   = new JLabel( "Etat: " );

  /**
   * Le timer qui commande la vitesse de l'excution
   */
  Timer delai;
  /************************************************************************************************************************************/


  /**
   * Le constructeur
   */
  public UIArchi()
  {
    super();

    GridBagLayout grille = new GridBagLayout();
    GridBagConstraints regles = new GridBagConstraints();
    setLayout( grille );
    isEtat.setForeground( Color.black );

    isEtat.setText( "Pas d'excution en cours -STOP-" );

    texte.add( etat );

    texte.add( isEtat );


    imageLabel.setVerticalAlignment( JLabel.CENTER );

    imageLabel.setHorizontalAlignment( JLabel.CENTER );

    image.setBackground( Color.white );

    texte.setBackground( Color.white );


    regles.fill = GridBagConstraints.BOTH;
    regles.anchor = GridBagConstraints.CENTER;

    GridUtils.buildConstraints( regles, new Insets( 0, 0, 0, 0 ), 1, 1, 0, 0, 1, 1, 100, 100 );

    grille.setConstraints( image, regles );

    add( image );


    GridUtils.buildConstraints( regles, new Insets( 0, 0, 0, 0 ), 0, 0, 0, 1, 1, 1, 100, 2 );

    grille.setConstraints( texte, regles );

    texte.setPreferredSize( getPreferredSize() );

    add( texte );


  }

  /**
   * Change la police et les icones pour le mode 1024-768
   */
  public void setBigIcons()
  {
    Font fb = new Font( "DIALOG", Font.PLAIN, 14 );
    Font fbb = new Font( "DIALOG", Font.BOLD, 14 );
    Font fpb = new Font( "DIALOG", Font.PLAIN, 14 );
    etat.setFont( fbb );
    isEtat.setFont( fb );
    image.setFont( fpb );
    imageLabel.setFont( fpb );
    image.setMode( 1024, 768 );
  }


  /**
   * Change la police et les icones pour le mode 800*600
   */
  public void setMedIcons()
  {
    Font fb = new Font( "DIALOG", Font.PLAIN, 12 );
    Font fbb = new Font( "DIALOG", Font.BOLD, 12 );
    Font fpb = new Font( "DIALOG", Font.PLAIN, 12 );
    etat.setFont( fbb );

    isEtat.setFont( fb );

    image.setFont( fpb );

    imageLabel.setFont( fpb );

    image.setMode( 800, 600 );

  }


  /**
   * Change la police et les icones pour le mode 640*480
   */
  public void setMiniIcons()
  {
    Font fb = new Font( "DIALOG", Font.PLAIN, 10 );

    Font fbb = new Font( "DIALOG", Font.BOLD, 10 );

    Font fpb = new Font( "DIALOG", Font.PLAIN, 9 );

    etat.setFont( fbb );

    isEtat.setFont( fb );

    image.setFont( fpb );

    imageLabel.setFont( fpb );

    image.setMode( 640, 480 );

  }

  /**
   * La methode qui definit le controleur
   * @ param ac : le controleur
   */
  public void setControler(ArchiControl ac)
  {
    control=ac;
    delai=new Timer( 5, control );
    delai.setInitialDelay( 5 );
    delai.setRepeats( false );
    image.setTimer( delai );
  }

  /**
   * La methode de redimensionnement de l'image
   * @param taille : une chaine codant pour l'une des 4 resolutions proposee
   */
  public void setImageSize( String taille )
  {
    i = new ImageIcon( "../Images/archi/archi" + taille + ".gif" );

    image.setPreferredSize( new Dimension(1,1) );

    setPreferredSize( image.getPreferredSize() );

    imageLabel.setIcon( i );

  }

  /**
   * Met a jour le texte d'tat de l'excution
   * @ param mesg : une chaine de carctres reprsentant le message a afficher
   */
  public void setMsg(String mesg)
  {
    isEtat.setText(mesg );
  }

  /**
   * Acces au champ texte
   * @return le champ de texte
   */
  public JLabel getMsg()
  {
    return isEtat;
  }

  /**
   * Acces a la reprsentation de l'architecture
   * @return le panel qui contient le dessin de l'architecture
   */
  public Visual getArchi()
  {
    return image;
  }


  /**
   *methode qui ajoute une animation a la liste
   *@return l'etat de l'animation
   */
  public void addAnimState(int anim){ image.addAnimState(anim );
 }

  /**
   * Methode qui dmarre les animations
   */
  public void startAnim()
  {
    image.startAnim();
  }

  /**
   * Met a 0 le n d'animation
   */
  public void resetColor()
  {
    image.resetColor();
  }

  /************************************************************************************************************************************/
  /**
   * La methode d'accs au timer
   * @return Le timer
   */
  public Timer getTimer()
  {
    return delai;
  }

  /**
   * La methode d'accs au label image
   * @return l'image du schema d'architecture
   */
  public JLabel getImageLabel()
  {
    return imageLabel;
  }

  /**
   * La methode qui change le dlai du timer
   * @param t : le nouveau dlai
   */
  public void setTime(int t)
  {
    delai.setDelay(t );
    delai.setInitialDelay(t );
  }

  /**
   * Dbute l'animation du bus 1
   */
  public void startAnimBus1()
  {
    image.startBus1Anim();
  }

  /**
   * Dbute l'animation du bus 1
   */
  public void animBus1()
  {

    image.bus1Anim();
  }

  /**
   * Dbute l'animation du bus 2
   */
  public void startAnimBus2()
  {
    image.startBus2Anim();
  }

  /**
   * Dbute l'animation du bus 2
   */
  public void animBus2()
  {
    image.bus2Anim();
  }


  /**
   * Dbute l'animation du bus 3
   */
  public void startAnimBus3()
  {
    image.startBus3Anim();
  }

  /**
   * Dbute l'animation du bus 3
   */
  public void animBus3()
  {
    image.bus3Anim();
  }

  /**
   * Dbute l'animation du bus 3 au registres
   */
  public void startAnimBus3Reg()
  {
    image.startBus3RegAnim();
  }

  /**
   * Dbute l'animation du bus 3 aux registres
   */
  public void animBus3Reg()
  {
    image.bus3RegAnim();
  }

  /**
   * Dbute l'animation du  registres au bus2
   */
  public void startAnimRegBus2()
  {
    image.startRegBus2Anim();
  }

  /**
   * Dbute l'animation du registres au bus2
   */
  public void animRegBus2()
  {
    image.regBus2Anim();
  }

  /**
   * Dbute l'animation du  registres au bus1
   */
  public void startAnimRegBus1()
  {
    image.startRegBus1Anim();
  }

  /**
   * Dbute l'animation du registres au bus1
   */
  public void animRegBus1()
  {
    image.regBus1Anim();
  }

  /**
   * Dbute l'animation du  bus1 a la memoire
   */
  public void startAnimBus1Mem()
  {
    image.startBus1MemAnim();
  }

  /**
   * Dbute l'animation du  bus1 a la memoire
   */
  public void animBus1Mem()
  {
    image.bus1MemAnim();
  }


  /**
   * Dbute l'animation du  bus2 a la memoire
   */
  public void startAnimBus2Mem()
  {
    image.startBus2MemAnim();
  }

  /**
   * Dbute l'animation du  bus2 a la memoire
   */
  public void animBus2Mem()
  {
    image.bus2MemAnim();
  }


  /**
   * Dbute l'animation de la memoire au bus 3
   */
  public void startAnimMemBus3()
  {
    image.startMemBus3Anim();
  }

  /**
   * Dbute l'animation de la memoire au bus 3
   */
  public void animMemBus3()
  {
    image.memBus3Anim();
  }


  /**
   * Dbute l'animation du  bus1 a l'ual
   */
  public void startAnimBus1Ual()
  {
    image.startBus1UalAnim();
  }

  /**
   * Dbute l'animation du  bus1 al'ual
   */
  public void animBus1Ual()
  {
    image.bus1UalAnim();
  }


  /**
   * Dbute l'animation du  bus2 a l'ual
   */
  public void startAnimBus2Ual()
  {
    image.startBus2UalAnim();
  }

  /**
   * Dbute l'animation du  bus2 a l'ual
   */
  public void animBus2Ual()
  {
    image.bus2UalAnim();
  }


  /**
   * Dbute l'animation de  l'ual au bus 3
   */
  public void startAnimUalBus3()
  {
    image.startUalBus3Anim();
  }

  /**
   * Dbute l'animation de  l'ual au bus 3
   */
  public void animUalBus3()
  {
    image.ualBus3Anim();
  }

  /**
   * Dbute l'animation du bus 3 au registre d'instructions
   */
  public void startAnimBus3RegI()
  {
    image.startBus3RegIAnim();
  }

  /**
   * Dbute l'animation du bus 3 au registre d'instructions
   */
  public void animBus3RegI()
  {
    image.bus3RegIAnim();
  }

  /**
   * Dbute l'animation du registre d'instructions au bus2
   */
  public void startAnimRegIBus2()
  {
    image.startRegIBus2Anim();
  }

  /**
   * Dbute l'animation du registre d'instructions au bus2
   */
  public void animRegIBus2()
  {
    image.regIBus2Anim();
  }

  /**
   * Dbute l'animation du bus 3 au radm
   */
  public void startAnimBus3Radm()
  {
    image.startBus3RadmAnim();
  }

  /**
   * Dbute l'animation du bus 3 au radm
   */
  public void animBus3Radm()
  {
    image.bus3RadmAnim();
  }

  /**
   * Dbute l'animation du radm au bus 1
   */
  public void startAnimRadmBus1()
  {
    image.startRadmBus1Anim();
  }

  /**
   * Dbute l'animation du radm au bus1
   */
  public void animRadmBus1()
  {
    image.radmBus1Anim();
  }

  /**
   * Dbute l'animation du bus 3 au cpt
   */
  public void startAnimBus3Cpt()
  {
    image.startBus3CptAnim();
  }

  /**
   * Dbute l'animation du bus 3 au cpt
   */
  public void animBus3Cpt()
  {
    image.bus3CptAnim();
  }

  /**
   * Dbute l'animation du cpt au bus 2
   */
  public void startAnimCptBus1()
  {
    image.startCptBus1Anim();
  }

  /**
   * Dbute l'animation du cpt au bus2
   */
  public void animCptBus1()
  {
    image.cptBus1Anim();
  }

  /**
   * Dbute l'animation du cpt au +2
   */
  public void startAnimCptCptPlus()
  {
    image.startCptCptPlusAnim();
  }

  /**
   * Dbute l'animation du cpt au +2
   */
  public void animCptCptPlus()
  {
    image.cptCptPlusAnim();
  }

  /**
   * Dbute l'animation du +2 au cpt
   */
  public void startAnimCptPlusCpt()
  {
    image.startCptPlusCptAnim();
  }

  /**
   * Dbute l'animation du +2 au cpt
   */
  public void animCptPlusCpt()
  {
    image.cptPlusCptAnim();
  }

  /**
   * Methode pour les animations
   */
  public void animate()
  {
    switch( image.getAnimState() )
    {
    case (PixConstants.BUS1ANIM):         animBus1();       break; // animation du bus 1
    case (PixConstants.BUS2ANIM):         animBus2();       break; // animation du bus 2
    case (PixConstants.BUS3ANIM):         animBus3();       break; // animation du bus 3
    case (PixConstants.REGTOBUS1ANIM):    animRegBus1();    break; // animation du registre au bus 1
    case (PixConstants.REGTOBUS2ANIM):    animRegBus2();    break; // animation du registre au bus 2
    case (PixConstants.BUS3TOREGANIM):    animBus3Reg();    break; // animation du bus 3 au registre
    case (PixConstants.BUS1TOUALANIM):    animBus1Ual();    break; // animation du bus 1 a l'UAL
    case (PixConstants.BUS2TOUALANIM):    animBus2Ual();    break; // animation du bus 2 a l'UAL
    case (PixConstants.UALTOBUS3ANIM):    animUalBus3();    break; // animation de l'UAL au bus 3
    case (PixConstants.BUS3TOREGIANIM):   animBus3RegI();   break; // animation du bus 3 au Registre d'Instruction
    case (PixConstants.REGITOBUS2ANIM):   animRegIBus2();   break; // animation du registre d'Instruction au bus 2
    case (PixConstants.BUS3TOCPTANIM):    animBus3Cpt();    break; // animation du bus 3 au PC
    case (PixConstants.CPTTOBUS1ANIM):    animCptBus1();    break; // animation du PC au bus 1
    case (PixConstants.CPTTOCPTPLUSANIM): animCptCptPlus(); break; // animation du PC a l'incrementeur
    case (PixConstants.CPTPLUSTOCPTANIM): animCptPlusCpt(); break; // animation du l'incrementeur au PC
    case (PixConstants.BUS3TORADMANIM):   animBus3Radm();   break; // animation du bus 3 au radm
    case (PixConstants.RADMTOBUS1ANIM):   animRadmBus1();   break; // animation du radm au bus 3
    case (PixConstants.BUS1TOMEMANIM):    animBus1Mem();    break; // animation du bus 1 a la memoire
    case (PixConstants.BUS2TOMEMANIM):    animBus2Mem();    break; // animation du bus 2 a la memoire
    case (PixConstants.MEMTOBUS3ANIM):    animMemBus3();    break; // animation de la memoire au bus 3

    default: System.out.println("Attention erreur inattendue!" ); break;
    }
  }

  /**
   * Stoppe l'animation en cours
   */
  public void stopAnim()
  {
    delai.stop();
    image.stopAnim();
  }


  /**
   * Methode qui poursuit l'execution lorsqu'une animation est finie
   */
  public void continueExec()
  {
    control.continueExec();

  }


  /**
   * La methode qui dit si la lecture d'une instruction doit tre anime ou non
   */
  public void setAnimLect(boolean b)
  {
    control.setAnimLect(b );
  }

  /************************************************************************************************************************************/
}
