/**
 * Le modele de table pour la fenetre d'edition
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IEdit;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.Dimension;
import java.util.Vector;

import Interface.IEdit.UIEdit;


public class EditDataModel extends AbstractTableModel
{
    /**
     *La fenetre contenant le tableau d'instructions assembleur
     */
    private UIEdit ui;
    /**
     *Une variable contenant le nom des colonnes
     */
    private String[] columnNames;
    /**
     *Le contenu du tableau d'instructions assembleurs
     */
    private Vector content;
    /**
     * le numero de la colonne editable
     */
    private int colonneEditable=1;


    /**
     *Le constructeur
     * @param uiEdit : la fenetre contenant la JTable ayant cette classe pour modele
     * @param header : les noms des colonnes du tableau
     * @param contenu: le contenu du tableau
     */
    public EditDataModel(UIEdit uiEdit,String[] header, Vector contenu )
    {
  columnNames=header;
  content=contenu;
  ui=uiEdit;
    }

    /**
     *Une methode a redefinir obligatoirement mais inutile ici
     * @see javax.swing.table.AbstractTableModel#getColumnCount()
     */
    public int getColumnCount() { return columnNames.length; }

    /**
     *Une methode a redefinir obligatoirement mais inutile ici
     * @see javax.swing.table.AbstractTableModel#getRowCount()
     */
    public int getRowCount() { return content.size();
}

    /**
     * Une methode a redefinir obligatoirement.
     * Elle remplit la table a chaque reaffichage de celle ci
     * @see javax.swing.table.AbstractTableModel#getValueAt(int row, int col)
     */
    public Object getValueAt(int row, int col)
    {
  if (col==0) {return new Integer(row+1 );
}
  else {return ((String)content.elementAt(row) );
}
    }

    /**
     * Une methode a redefinir obligatoirement.
     * Elle renvoie le nom de la colonne
     * @see javax.swing.table.AbstractTableModel#getColumnName(int column)
     */
    public String getColumnName(int column) {return columnNames[column];}

   /**
     * Une methode a redefinir obligatoirement.
     * Elle determine quelle partie du tableau est editable
     * @see javax.swing.table.AbstractTableModel#isCellEditable(int row, int col)
     */
    public boolean isCellEditable(int row, int col) {return (col==colonneEditable );
}

    /**
     * La methode qui determine quelle colonne est editable
     * @param x : la colonne editable, -1 si aucune n'est editable
     */
    public void setColonneEditable(int x) { colonneEditable=x;}

    /**
     * Une methode a redefinir obligatoirement.
     * Elle permet de rentrer manuellemnt une valeur dans la table
     * @see javax.swing.table.AbstractTableModel#setValueAt(Object aValue, int row, int col)
     */
    public void setValueAt(Object aValue, int row, int col)
    {
  if (content.elementAt(row).equals(""))
      content.addElement("" );
  content.setElementAt((String)aValue,row );
  ui.setMEditAt((String)aValue,row );
    }

}


