/**
 * La partie de l'interface  gerant l'editeur d'instructions assembleur.
 * Elle contient la table o sont rangees les instructions et des boutons Charger, Sauver et Editer.
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IEdit;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import javax.accessibility.*;

import Utilitaires.GridUtils;
import Controlers.EditControl;


public class UIEdit extends JPanel

{
  /**
   * Le controleur d'edition
   */
  private EditControl mControl;

  /**
   * l'editeur externe
   */
  private EditExt mEditor;

  /**
   * Une boite de dialogue swing pour le chargement et la sauvegarde des fichiers
   */
  private JFileChooser mFileChoice = new JFileChooser();

  /**
   * boite de dialogiue d'erreur lex/syn
   */
  private JDialog mErrorDialog;

  /**
   * Le bouton mOKBtn de la boite de dialogue
   */
  private JButton mOKBtn;

  /**
   * Le texte de la boite de dialogue
   */
  private ImageIcon mSaveIcn   = new ImageIcon( "../Images/icons/normal/save.gif" );
  private ImageIcon mLoadIcn   = new ImageIcon( "../Images/icons/normal/open.gif" );
  private ImageIcon mEditIcn   = new ImageIcon( "../Images/icons/normal/edit.gif" );
  private ImageIcon mSmSaveIcn = new ImageIcon( "../Images/icons/mini/MiniSave.gif" );
  private ImageIcon mSmLoadIcn = new ImageIcon( "../Images/icons/mini/MiniOpen.gif" );
  private ImageIcon mSmEditIcn = new ImageIcon( "../Images/icons/mini/MiniEdit.gif" );

  private JPanel  mButtons = new JPanel();
  private JButton mLoadBtn = new JButton( "Charger", mLoadIcn );
  private JButton mSaveBtn = new JButton( "Sauver",  mSaveIcn );
  private JButton mEditBtn = new JButton( "Editer",  mEditIcn );
  private JLabel  mTitle   = new JLabel( "Editeur :", JLabel.CENTER );
  private JLabel  mPlace;

  private JScrollPane sPane;

  /**
    * Le modele de tableau d'edition
    */
  private EditDataModel mDataModel;
  private JTable mTable;
    /**
    * Les noms des colonnes
    */
  private Vector mContent;

  /**
    * le constructeur
    */
  public UIEdit()
  {
    super();
    
    mLoadBtn.setToolTipText("Permet le chargement d'un fichier contenant des instructions assembleur" );
    mSaveBtn.setToolTipText("Permet l'enregistrement des instructions assembleur dans un fichier" );
    mEditBtn.setToolTipText("Ouvre une fentre ddie a l'dition d'instructions" );
    
    GridBagLayout lGrid = new GridBagLayout();
    GridBagConstraints lRules = new GridBagConstraints();
    
    setLayout( lGrid );
    lRules.fill = GridBagConstraints.BOTH;
    lRules.anchor = GridBagConstraints.CENTER;
    Insets lLimits = new Insets( 1, 1, 1, 1 );
    
    GridBagLayout lGrid2 = new GridBagLayout();
    GridBagConstraints lRules2 = new GridBagConstraints();
    
    mButtons.setLayout( lGrid2 );
    lRules2.fill = GridBagConstraints.BOTH;
    lRules2.anchor = GridBagConstraints.CENTER;
    Insets lLimits2 = new Insets( 0, 0, 0, 0 );

    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 0, 1, 1, 100, 150 );
    lGrid2.setConstraints( mTitle, lRules2 );
    mTitle.setPreferredSize( mButtons.getPreferredSize() );
    mButtons.add( mTitle );

    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 1, 1, 1, 100, 100 );
    lGrid2.setConstraints( mLoadBtn, lRules2 );
    mLoadBtn.setPreferredSize( mButtons.getPreferredSize() );
    mButtons.add( mLoadBtn );

    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 2, 1, 1, 100, 100 );
    lGrid2.setConstraints( mSaveBtn, lRules2 );
    mSaveBtn.setPreferredSize( mButtons.getPreferredSize() );
    mButtons.add( mSaveBtn );
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 3, 1, 1, 100, 100 );
    lGrid2.setConstraints( mEditBtn,lRules2 );
    mEditBtn.setPreferredSize( mButtons.getPreferredSize() );
    mButtons.add(mEditBtn );
    
    mButtons.setPreferredSize(getPreferredSize() );
    GridUtils.buildConstraints(lRules, lLimits, 0, 0, 0, 0, 1, 1, 65, 100 );
    lGrid.setConstraints(mButtons, lRules );
    add(mButtons );

    sPane = new JScrollPane();

    sPane.setPreferredSize(getPreferredSize() );
    GridUtils.buildConstraints(lRules, lLimits, 0, 0, 1, 0, 1, 1, 100, 100 );
    lGrid.setConstraints(sPane, lRules );
    add(sPane );
  }


  /**
    * Change la police et les icones pour le mode 1024*768
    */
  public void setBigIcons()
  {
    Font lFontB = new Font("DIALOG", Font.PLAIN, 14 );
    mLoadBtn.setDisabledIcon( mLoadIcn );
    mLoadBtn.setIcon( mLoadIcn );
    mLoadBtn.setFont( lFontB );
    mSaveBtn.setDisabledIcon( mSaveIcn );
    mSaveBtn.setIcon( mSaveIcn );
    mSaveBtn.setFont( lFontB );
    mEditBtn.setDisabledIcon( mEditIcn );
    mEditBtn.setIcon( mEditIcn );
    mEditBtn.setFont( lFontB );
    mTitle.setFont( new Font("DIALOG", Font.BOLD, 14 ) );
  }


  /**
    * Change la police et les icones pour le mode 800*600
    */
  public void setMedIcons()
  {
    Font lFontB = new Font("DIALOG", Font.PLAIN, 12 );
    mLoadBtn.setDisabledIcon( mLoadIcn );
    mLoadBtn.setIcon( mLoadIcn );
    mLoadBtn.setFont( lFontB );
    mSaveBtn.setDisabledIcon( mSaveIcn );
    mSaveBtn.setIcon( mSaveIcn );
    mSaveBtn.setFont( lFontB );
    mEditBtn.setDisabledIcon( mEditIcn );
    mEditBtn.setIcon( mEditIcn );
    mEditBtn.setFont( lFontB );
    mTitle.setFont( new Font("DIALOG", Font.BOLD, 12 ) );
  }


  /**
    * Change la police et les icones pour le mode 640*480
    */
  public void setMiniIcons()
  {
    Font lFontP = new Font("DIALOG", Font.PLAIN, 10 );
    mLoadBtn.setDisabledIcon( mSmLoadIcn );
    mLoadBtn.setIcon( mSmLoadIcn );
    mLoadBtn.setFont( lFontP );
    mSaveBtn.setDisabledIcon( mSmSaveIcn );
    mSaveBtn.setIcon( mSmSaveIcn );
    mSaveBtn.setFont( lFontP );
    mEditBtn.setDisabledIcon( mSmEditIcn );
    mEditBtn.setIcon( mSmEditIcn );
    mEditBtn.setFont( lFontP );
    mTitle.setFont( new Font("DIALOG", Font.BOLD, 10 ) );
  }


  /**
    * La methode qui definit le controleur
    * @ param ec : le controleur
    */
  public void setControler( EditControl inEditCtrl ) {  mControl = inEditCtrl; }

  /**
    * La methode qui ajoute l'ecouteur d'evenement aux composants qui en ont besoin
    * et qui cree le tableau qui avait besoin au prealable du controleur
    */
  public void setData()
  {
    mControl.createListener();
    mContent = new Vector();
    
    for(int i=0;i<15;i++)
      mContent.addElement( "" );
      
    String [] lColumnNames = {"N", "INSTRUCTION"};
    mDataModel = new EditDataModel( this, lColumnNames, mContent );
    mTable = new JTable(mDataModel );

    mTable.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn lNumColumn = mTable.getColumn("N" );
    lNumColumn.setPreferredWidth( 40 );
    TableColumn lInstrColumn = mTable.getColumn("INSTRUCTION" );
    lInstrColumn.setPreferredWidth( 200 );

    mTable.setPreferredScrollableViewportSize(
      new Dimension((int)sPane.getPreferredSize().getWidth(),
      (int)sPane.getPreferredSize().getHeight())
    );
    sPane.getViewport().add(mTable );
  }

  /**
    * La methode qui cree une boite de dialogue d'erreur
    */
  public void createErrorDialog()
  {
    JFrame lParent = (JFrame)this.getTopLevelAncestor();
    mPlace = new JLabel( "", new ImageIcon("../Images/icons/normal/assemb.gif" ), JLabel.CENTER );
    mPlace.setFont( mTitle.getFont() );

    mTable.revalidate();
    mTable.repaint();

    mErrorDialog = new JDialog( lParent, "Avertissement !", true );
    mErrorDialog.setSize( 25*lParent.getWidth()/32, lParent.getHeight()/4 );
    mErrorDialog.setLocation(
      ((int)lParent.getLocation().getX()) + ((lParent.getWidth() - mErrorDialog.getWidth())/2),
      ((int)lParent.getLocation().getY()) + ((lParent.getHeight() - mErrorDialog.getHeight())/2)
    );

    mErrorDialog.getContentPane().setLayout( new GridLayout(2, 1) );
    JPanel lPane = new JPanel();
    mOKBtn = new JButton("Ok", new ImageIcon("../Images/icons/normal/Ok.gif" ) );
    mOKBtn.setBorder( new SoftBevelBorder(SoftBevelBorder.RAISED) );
    mOKBtn.setFont( mTitle.getFont() );
    mControl.createListenerForError();
    lPane.add( mOKBtn );
    mErrorDialog.getContentPane().add( mPlace );
    mErrorDialog.getContentPane().add( lPane );
    mErrorDialog.setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
  }

  /**
    * La methode qui annonce une erreur de syntaxe dans le programme source
    * @param s : le numero de ligne de l'erreur
    */
  public void showErrorSyntax(String s)
  {
    mTable.revalidate();
    mTable.repaint();
    createErrorDialog();
    mPlace.setText("Erreur syntaxique a la ligne "+s+" du programme" );
  
    mErrorDialog.show();
    mErrorDialog.setModal( true );
  }

  /**
    * La methode qui annonce une erreur de format dans le programme source
    * @param s : le numero de ligne de l'erreur
    */
  public void showErrorFormat(String s)
  {
    setSelected(Integer.valueOf(s).intValue() );
    mTable.revalidate();
    mTable.repaint();
    createErrorDialog();
    mPlace.setText("Erreur de format a la ligne "+s+" du programme" );
  
    mErrorDialog.show();
    mErrorDialog.setModal( true );
  }

  /**
    * La methode qui annonce une erreur de saut (en dehors du programme) dans le programme source
    * @param s : le numero de ligne de l'erreur
    */
  public void showErrorJump(String s)
  {
    createErrorDialog();
    mPlace.setText("Le saut vers l'instruction "+s+" est hors des limites du programme" );
    mErrorDialog.show();
    mErrorDialog.setModal( true );
  }


  /**
    * La methode d'acces au bouton mLoadBtn
    * @return le bouton precite
    */
  public JButton getLoadBtn() { return mLoadBtn; }

  /**
    * La methode d'acces au bouton mSaveBtn
    * @return le bouton precite
    */
  public JButton getSaveBtn() { return mSaveBtn; }

  /**
    * La methode d'acces au bouton mEditBtn
    * @return le bouton precite
    */
  public JButton getEditBtn() { return mEditBtn; }

  /**
    * La methode d'acces au contenu de l'editeur
    * @return le Vector mContent
    */
  public Vector getContent() { return mContent; }

  /**
    * La methode d'acces a l'editeur externe
    * @return mEditor
    */
  public EditExt getEditExt() { return mEditor; }

   /**
     * La methode d'acces a la boite d'erreur
     * @return mErrorDialog
     */
    public JDialog getError() { return mErrorDialog; }

  /**
    * La methode d'acces a la teble
    * @return mTable
    */
  public JTable getTable() { return mTable; }

  /**
    * La methode d'acces au bouton mOKBtn de la boite d'erreur
    * @return bouton mOKBtn
    */
  public JButton getErrorOk() { return mOKBtn; }

  /**
    * La methode d'acces a la boite de dialogue de sauvegarde/chargement
    * @return mFileChoice
    */
  public JFileChooser getFileChoice() { return mFileChoice; }

  /**
    * La methode qui permet de remplacer le contenu de la JTable par une autre suite d'instructions assembleur
    * @param contenu : le Vector contenant les nouvelles instructions a afficher
    */
  public void setContent(Vector contenu)
  {
    mContent = (Vector)contenu.clone();
    for( int i = 0; i < 15; i++ )
      mContent.addElement( "" );
    String [] lColumnNames = { "N", "INSTRUCTION" };
    mDataModel = new EditDataModel( this, lColumnNames, mContent );
    mTable = new JTable( mDataModel );

    mTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn lNumColumn = mTable.getColumn("N" );
    lNumColumn.setPreferredWidth( 40 );
    TableColumn lInstrColumn = mTable.getColumn("INSTRUCTION" );
    lInstrColumn.setPreferredWidth(200 );

    mTable.setPreferredScrollableViewportSize(
      new Dimension((int)sPane.getPreferredSize().getWidth(),
      (int)sPane.getPreferredSize().getHeight())
    );
    sPane.getViewport().add( mTable );
  }

  /**
    * La methode qui permet de remplacer le contenu de la JTable a un endroit par une autre instruction assembleur
    * @param val : la nouvelle instruction
    * @param row : la ligne du tableau ou la mettre
    */
  protected  void setMEditAt( String inStr, int lRow ) { mControl.setMEditAt( inStr, lRow ); }

  /**
    * La methode qui cre la fenetre d'editeur externe
    * Elle est appele par EditControl lorsque l'utilisateur clique sur le bouton "editeur"
    * @see Controlers.EditControl
    */
  public void showEditor( String inTxt ) {
    mEditor = new EditExt( this, inTxt );
    mEditor.setControler( mControl );
    mControl.createListenerForEditor();
    mEditor.setVisible( true );
  }

  /**
    * Methode qui permet de selectionner une ligne dans le tableau
    */
  public void setScrollBar( int inIndex )
  {
    JFrame lFrame = (JFrame)getTopLevelAncestor();
    lFrame.repaint(
      (27*lFrame.getWidth())/40,
      (25*lFrame.getHeight())/40,
      (13*lFrame.getWidth())/40,
      (15*lFrame.getHeight())/40
    );
    sPane.getVerticalScrollBar().setValue(
      (sPane.getVerticalScrollBar().getMaximum() / mContent.size()) * (inIndex-3)
    );
  }

  /**
    * Methode qui permet de selectionner une ligne dans le tableau
    */
  public void setSelected( int inIndex )
  {
    mTable.clearSelection();
    mTable.addRowSelectionInterval( inIndex-1, inIndex-1 );
    mTable.editCellAt( inIndex-1, 1 );
    setScrollBar( inIndex );
  }

  /**
    * Change l'accessibilite
    * @param inAccess: l'accessibilite
    */
  public void setEditable( boolean inAccess ) {
    if( inAccess )
      mDataModel.setColonneEditable( 1 );
    else {
      mTable.editCellAt( 0, 0 );
      mTable.clearSelection();
      mDataModel.setColonneEditable( -1 );
    }
    mLoadBtn.setEnabled( inAccess );
    mSaveBtn.setEnabled( inAccess );
    mEditBtn.setEnabled( inAccess );
  }

  /**
    * La methode qui permet de changer des valeurs l'diteur
    */
  public void setAllowed()
  {
    mDataModel.setColonneEditable(1 );
    mLoadBtn.setEnabled( true );
    mSaveBtn.setEnabled( true );
    mEditBtn.setEnabled( true );
  }
  
  /**
    * La methode qui interdit de changer des valeurs dans l'diteur
    */
  public void setForbidden()
  {
    mTable.editCellAt(0, 0 );
    mTable.clearSelection();
    mDataModel.setColonneEditable(-1 );
    mLoadBtn.setEnabled( false );
    mSaveBtn.setEnabled( false );
    mEditBtn.setEnabled( false );
  }

  /**
    * Methode qui permet de stopper les editions du tableau
    */
  public void unEdit()
  {
    int i = mTable.getEditingRow();
    int j = mTable.getEditingColumn();
    mTable.editCellAt( 0, 0 );
    if ( i != -1 && j != -1 ) {
      String txt = (String)mTable.getValueAt(i, j );
      if (txt==null || txt.equals("")) {
        /*control.removeLine(i );*/
        setMEditAt("", i );
      }
    }
  }

  /**
    * Methode qui permet de deselectionner tout tableau
    */
  public void unSelectAll() { mTable.clearSelection(); }

  /**
    * Methode servant a trouver le numero de ligne d'unze instruction errone
    * @param l : le numero de la ligne sans tenir compte des espaces dans l'editeur
    * @return le numero de ligne en tenant compte
    */
  public int seekPos( int l )
  {
    int cpt=0;
    int cpt2=0;
    while (cpt!=l) {
      if( mTable.getValueAt(cpt2, 1).equals("") ||
          ((((String)mTable.getValueAt(cpt2, 1)).substring(0, 1)).equals(" ")) )
        cpt2++;
      else {
        cpt++;
        cpt2++;
      }
    }
    return cpt2;
  }

  /**
    * Methode d'acces au controleur
    */
  public EditControl getControl() { return mControl; }
}



