/**
 * Le modele de table pour la fenetre de la RAM
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IRam;

import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

import Interface.IRam.UIRam;
import Utilitaires.*;

public class RamDataModel extends AbstractTableModel
{
  /**
    *La fenetre contenant le tableau de la RAM
    */
  private UIRam mInterface;
  /**
    *Le contenu du tableau de RAM
    */
  private Vector mStorage;
  /**
    * Les numeros des cases a afficher
    */
  private Vector mShownAddr;
  /**
    *Une variable contenant le nom des colonnes
    */
  private String[] mColNames;
  /**
    * le numero de la colonne editable
    */
  private int mMutableCol = 1;
  /**
    * le type d'affichage du contenu: binaire(defaut) decimal ou hexadecimal
    */
  private int mViewType = PixConstants.BINAIRE;

  /**
    * Le constructeur
    * @param inInterface : la fenetre contenant la JTable ayant cette classe pour modele
    * @param inStorage : le contenu du tableau
    * @param inShownAddr : les adresses des cases affichees
    * @param inColNames : les noms des colonnes du tableau
    */
  public RamDataModel( UIRam inInterface, Vector inStorage, Vector inShownAddr, String[] inColNames ) {
    mInterface = inInterface;
    mStorage   = inStorage;
    mShownAddr = inShownAddr;
    mColNames  = inColNames;
  }

  /**
    * Une methode a redefinir obligatoirement mais inutile ici
    * @see javax.swing.table.AbstractTableModel#getColumnCount()
    */
  public int getColumnCount() { return mColNames.length; }

  /**
    * Une methode a redefinir obligatoirement mais inutile ici
    * @see javax.swing.table.AbstractTableModel#getRowCount()
    */
  public int getRowCount() { return mStorage.size(); }

  /**
    * Une methode a redefinir obligatoirement.
    * Elle remplit la table a chaque reaffichage de celle ci
    * @see javax.swing.table.AbstractTableModel#getValueAt(int inRow, int inCol)
    */
  public Object getValueAt( int inRow, int inCol ) {
    if( inCol == 0 )
      return ((Integer)mShownAddr.elementAt(inRow) );
    else {
      // a priori, (inCol == 1)
      switch ( mViewType ) {
      case (PixConstants.DECIMAL):
        return Integer.toString( ((Integer)mStorage.elementAt( inRow )).intValue() );
      case (PixConstants.BINAIRE):
        return Utils.unsignedExtension(
          Utils.signedIntToBin(((Integer)mStorage.elementAt(inRow)).intValue()), 16 );
      case (PixConstants.HEXADECIMAL):
        return Utils.setToNuHexChars(
          Utils.signedIntToHex(((Integer)mStorage.elementAt(inRow)).intValue()), 4 );
      default: return "Erreur impossible";
      }
    }
  }

  /**
    * Une methode a redefinir obligatoirement.
    * Elle renvoie le nom de la colonne
    * @see javax.swing.table.AbstractTableModel#getColumnName(int inCol)
    */
  public String getColumnName( int inCol ) { return mColNames[inCol]; }

  /**
    * Une methode a redefinir obligatoirement.
    * Elle determine quelle partie du tableau est editable
    * @see javax.swing.table.AbstractTableModel#isCellEditable(int inRow, int inCol)
    */
  public boolean isCellEditable( int inRow, int inCol ) { return (inCol == mMutableCol); }

  /**
    * Une methode a redefinir obligatoirement.
    * Elle permet de rentrer manuellemnt une valeur dans la table
    * @see javax.swing.table.AbstractTableModel#setValueAt(Object aValue, int inRow, int inCol)
    */
  public void setValueAt( Object aValue, int inRow, int column ) {
    try {
      Integer lValue = new Integer( 0 );
      switch( mViewType) {
      case PixConstants.DECIMAL:
        lValue = new Integer((String)aValue );
        if( lValue.intValue() < -32768 || lValue.intValue() > 32767 )
          throw new NumberFormatException( "" );
        break;
      case PixConstants.BINAIRE:
        if(((String)aValue).length() > 16 )
          throw new NumberFormatException( "" );
        lValue = new Integer( Utils.signedBinToInt( Utils.unsignedExtension( (String)aValue, 16 ) ) );
        break;
      case PixConstants.HEXADECIMAL:
        if(((String)aValue).length() > 4 )
          throw new NumberFormatException("" );
        lValue = new Integer(Utils.signedHexToInt(Utils.setToNuHexChars((String)aValue, 4), 16) );
        break;
      }
      mStorage.setElementAt( lValue, inRow );
      mInterface.setRamFromUI( lValue, ((Integer)mShownAddr.elementAt(inRow)).intValue() );
    } catch (NumberFormatException e) {
      /***Boite de dialogue d'erreur de format a ouvrir ici***/
    }
  }

  /**
    * La methode qui change le type d'affichage du contenu de la RAM
    * @param type : le nouveau type d'affichage
    */
  public void changeTypeAffichage( int inType ) { mViewType = inType; /* filltab(); */ }

  /**
    * La methode qui determine quelle colonne est editable
    * @param x : la colonne editable, -1 si aucune n'est editable
    */
  public void setColonneEditable( int inCol ) { mMutableCol = inCol; }

  /**
    * La methode d'acces au type d'affichage
    * @return binaire,decimel ou hexa
    */
  public int getDisplayType() { return mViewType; }
}
