/**
 * La partie de l'interface  gerant la fenetre d'affichage de la RAM.
 * Elle contient la table o est stockee le contenu de la RAM
 * et des checkbox pour le mode de visualisation de la RAM
 * et le type du contenu binaire decimal ou hexadecimal.
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IRam;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.Vector;
import javax.swing.border.*;
import java.awt.event.*;
import javax.accessibility.*;

import Controlers.RamControl;
import Utilitaires.*;

public class UIRam extends JPanel
{
  /**
    * Le modele du tableau de la RAM
    */
  private  RamDataModel mDataModel;

  /**
    * La memoire
    */
  private  Vector mMemory;
  private  Vector mMemAddr;

  /**
    * Une structure pour stocker les cases modifiees
    */
  private Vector mAlteredAddr = new Vector();
  private Vector mAlteredVals = new Vector();

  /**
    * Une structure pour stocker les cases choisies
    */
  private Vector mUserAddr = new Vector();
  private Vector mUserVals = new Vector();

  /**
    * Une variable qui donne le mode : true pour novice,  false pour expert
    */
  boolean mIsBeginer = true;

  /**
    * Le controleur de la RAM
    */
  private RamControl mControl;

  /**
    * La fenetre de choix des cases de la RAM a voir
    */
  private RamChoiceFrame mChoiceFrame;


  private  JScrollPane sPane;
  private  JTable      mRamTable;
  private  JComboBox   mRamViewCombo   = new JComboBox();
  private  JComboBox   mViewTypeCombo  = new JComboBox();
  private  JLabel      mRamLabel       = new JLabel( "Mmoire :", JLabel.CENTER );
  private  JLabel      mTypeLabel      = new JLabel( "Reprsentation :" );
  private  JLabel      mRamChoiceLabel = new JLabel( "Type d'affichage :" );
  private  JPanel      mContentPane;
  private  String[]    mColsTab        = { "ADRESSE",  "CONTENU" };

  /**
    * le constructeur
    */
  public UIRam()
  {
    super();
    mRamViewCombo.setToolTipText("Permet de choisir la partie de la mmoire vive a afficher" );
    mViewTypeCombo.setToolTipText("Permet de choisir le type du contenu de la mmoire et des registres" );
  
    GridBagLayout lGrid = new GridBagLayout();
    GridBagConstraints lRules = new GridBagConstraints();
  
    setLayout(lGrid );
    lRules.fill = GridBagConstraints.BOTH;
    lRules.anchor = GridBagConstraints.CENTER;
  
    Insets lLimits = new Insets( 1, 1, 1, 1 );
  
    sPane = new JScrollPane();
    mContentPane = new JPanel();

    sPane.setPreferredSize( getPreferredSize() );
    mContentPane.setPreferredSize( getPreferredSize() );

    fillContentPane();

    GridUtils.buildConstraints( lRules, lLimits, 0, 0, 0, 0, 1, 1, 40, 100 );
    lGrid.setConstraints( mContentPane, lRules );
    add( mContentPane );

    GridUtils.buildConstraints( lRules, lLimits, 0, 0, 1, 0, 1, 1, 60, 100 );
    lGrid.setConstraints( sPane, lRules );
    add( sPane );
  }

  /**
    * Change la police et les icones pour le mode 1024*768
    */
  public void setBigIcons()
  {
    Font lFontB = new Font("DIALOG", Font.PLAIN, 14 );
    mRamChoiceLabel.setFont( lFontB );
    mTypeLabel.setFont( lFontB );
    mRamViewCombo.setFont( lFontB );
    mViewTypeCombo.setFont( lFontB );
    mRamLabel.setFont( new Font("DIALOG", Font.BOLD, 14 ) );
   }


  /**
    * Change la police et les icones pour le mode 800*600
    */
  public void setMedIcons()
  {
    Font lFontB = new Font("DIALOG", Font.PLAIN, 12 );
    mRamChoiceLabel.setFont(lFontB );
    mTypeLabel.setFont(lFontB );
    mRamViewCombo.setFont(lFontB );
    mViewTypeCombo.setFont(lFontB );
    mRamLabel.setFont( new Font("DIALOG", Font.BOLD, 12 ) );
  }


  /**
    * Change la police et les icones pour le mode 640*480
    */
  public void setMiniIcons()
  {
    Font fp = new Font("DIALOG", Font.PLAIN, 10 );
    mRamChoiceLabel.setFont(fp );
    mTypeLabel.setFont(fp );
    mRamViewCombo.setFont(fp );
    mViewTypeCombo.setFont(fp );
    mRamLabel.setFont( new Font("DIALOG", Font.BOLD, 10 ) );
  }


  /**
    * une methode qui remplit le mContentPane avec ses combobox
    */
  private void fillContentPane()
  {
    mRamViewCombo.addItem("Tout" );
    mRamViewCombo.addItem("Modifi" );
    mRamViewCombo.addItem("Au choix" );

    mViewTypeCombo.addItem("Binaire" );
    mViewTypeCombo.addItem("Dcimal" );
    mViewTypeCombo.addItem("Hexadcimal" );
    
    
    GridBagLayout lGrid2 = new GridBagLayout();
    GridBagConstraints lRules2 = new GridBagConstraints();
    
    mContentPane.setLayout( lGrid2 );
    lRules2.fill = GridBagConstraints.BOTH;
    lRules2.anchor = GridBagConstraints.CENTER;
    Insets lLimits2 = new Insets( 0, 0, 0, 0 );
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 0, 1, 1, 100, 150 );
    lGrid2.setConstraints( mRamLabel, lRules2 );
    mRamLabel.setPreferredSize( mContentPane.getPreferredSize() );
    mContentPane.add( mRamLabel );
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 1, 1, 1, 100, 100 );
    lGrid2.setConstraints( mRamChoiceLabel, lRules2 );
    mRamChoiceLabel.setPreferredSize( mContentPane.getPreferredSize() );
    mContentPane.add( mRamChoiceLabel );
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 2, 1, 1, 100, 100 );
    lGrid2.setConstraints( mRamViewCombo, lRules2 );
    mRamViewCombo.setPreferredSize( mContentPane.getPreferredSize() );
    mContentPane.add( mRamViewCombo );
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 3, 1, 1, 100, 100 );
    lGrid2.setConstraints( mTypeLabel, lRules2 );
    mTypeLabel.setPreferredSize( mContentPane.getPreferredSize() );
    mContentPane.add( mTypeLabel );
    
    
    GridUtils.buildConstraints( lRules2, lLimits2, 0, 0, 0, 4, 1, 1, 100, 100 );
    lGrid2.setConstraints( mViewTypeCombo, lRules2 );
    mViewTypeCombo.setPreferredSize( mContentPane.getPreferredSize() );
    mContentPane.add( mViewTypeCombo );
  }

  /**
    * La methode qui definit le controleur
    * @ param rc : le controleur
    */
  public void setControler( RamControl inRamControl ) { mControl = inRamControl; }

  /**
    * La methode qui ajoute l'ecouteur d'evenement aux composants qui en ont besoin
    * et qui cree le tableau qui avait besoin au prealable du controleur
    */
  public void setData()
  {
    mControl.setRamInUI();
    mControl.createListener();

    mMemAddr = new Vector(65536 );
    for( int i = 0; i < 65536; i++)
      mMemAddr.addElement(new Integer(i) );

    mDataModel = new RamDataModel( this, mMemory, mMemAddr, mColsTab );

    mRamTable = new JTable( mDataModel );
    mRamTable.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn lAddrCol = mRamTable.getColumn( "ADRESSE" );
    lAddrCol.setPreferredWidth( 100 );
    TableColumn lValsCol = mRamTable.getColumn( "CONTENU" );
    lValsCol.setPreferredWidth( 150 );
    
    mRamTable.setPreferredScrollableViewportSize( sPane.getPreferredSize() );
    sPane.getViewport().add( mRamTable );
    mRamViewCombo.setEnabled( true );
    mViewTypeCombo.setEnabled( true );
    setForbidden();
  }

  /**
    * La methode qui permet de remplacer le contenu d'une adresee du modele la RAM
    * @param inVal : le nouveau contenu
    * @param inAddr : l'adresse ou la mettre
    * @see Controlers.RamControl#setRamFromUI(Integer , int )
    */
  public void setRamFromUI(Integer inVal, int inAddr) {
    mMemory.setElementAt( inVal, inAddr );
    mControl.setRamFromUI( inVal, inAddr );
  }
  
  /**
    * La methode qui permet de remplacer le tableau par un nouveau
    * @param inStorage : le nouveau contenu
    */
  public void setNewRam( Vector inStorage, Vector inAddrs )
  {
    int lCurType = mDataModel.getDisplayType();
    sPane.getViewport().remove( mRamTable );
    mDataModel = null;
    mRamTable  = null;
    mDataModel = new RamDataModel( this, inStorage, inAddrs, mColsTab );
    mRamTable  = new JTable( mDataModel );
    setAffichage( lCurType );
    mRamTable.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn lAddrCol = mRamTable.getColumn( "ADRESSE" );
    lAddrCol.setPreferredWidth(100 );
    TableColumn lValsCol = mRamTable.getColumn( "CONTENU" );
    lValsCol.setPreferredWidth(150 );
    mRamTable.setPreferredScrollableViewportSize( sPane.getPreferredSize() );
    sPane.getViewport().add( mRamTable );
    if (mIsBeginer) setForbidden();
  }
  
  /**
    * La methode d'acces au modele du tableau
    * @return le modele de JTable
    */
  public RamDataModel getRamDataModel() { return mDataModel; }

  /**
    * La methode qui permet de remplir la RAM a l'affichage a partir du modele
    * @param inStorage : le contenu de la RAM(pris dans le modele sous forme d'un Vector)
    */
  public void setRam(Vector inStorage) { mMemory = (Vector)inStorage.clone(); }

  /**
    * La methode qui rafraichit la ram
    */
  public void refreshRam() {
    mRamTable.revalidate();
    mRamTable.repaint();
  }

  /**
    * La methode qui permet de remplacer le contenu d'une adresee dans l'affichage de la RAM
    * @param inVal : la nouvelle valeur
    * @param inAddr : l'adresse ou la mettre
    */
  public void setMemAt( Integer inVal, int inAddr )
  {
    mMemory.setElementAt( inVal, inAddr );
    mRamTable.revalidate();
    mRamTable.repaint();
  }
  
  /**
    * La methode d'acces au contenu des cases choisies
    * @return la RAM
    */
  public  Vector getUserVals() { return mUserVals; }

  /**
    * La methode d'acces au n des cases choisies
    * @return la RAM choisie
    */
  public  Vector getUserAddr() {return mUserAddr;}

  /**
    * La methode de mise a jour des cases choisies
    * @param inVal : la valeur a inserer
    * @param inAddr : la case ou inserer
    */
  public void setChoiceAt( Integer inVal, Integer inAddr ) {
    int lIndex = mUserAddr.indexOf( inAddr );
    if (lIndex!=-1)
      mUserVals.setElementAt( inVal, lIndex );
    if ( mControl.getRamType().equals("Au choix") )
      setNewRam( mUserVals, mUserAddr );
  }

  /**
    * La methode de visualisation des cases choisies
    * @param inVals : les valeurs
    * @param inAddrs : les adresses
    */
  public void setChoice( Vector inVals, Vector inAddrs ) {
    mUserVals = (Vector)( inVals.clone() );
    mUserAddr = (Vector)( inAddrs.clone() );
    setNewRam( inVals, inAddrs );
  }
  
  /**
    * La methode de visualisation des cases modifiees
    * @param inAddrs :  les cases memoire modifiees
    * @param inVals :  le contenu memoire modifie
    */
  public void setAlter( Vector inVals, Vector inAddrs )
  {
    mAlteredAddr=(Vector)inAddrs.clone();
    mAlteredVals=(Vector)inVals.clone();
    setNewRam(mAlteredVals, mAlteredAddr );
  }

  /**
    * La methode de visualisation des cases modifies
    */
  public void setAll() { setNewRam( getMem(), getMemAddr() ); }

  /**
    * La methode d'acces au  tableau de la RAM
    * @return la RAM
    */
  public  Vector getMem() { return mMemory; }

  /**
    * La methode d'acces a la teble
    * @return table
    */
  public JTable getTable() { return mRamTable; }

  /**
    * La methode d'acces aux index du tableau  de la RAM
    * @return mMemAddr
    */
  public  Vector getMemAddr() { return mMemAddr; }

  /**
    * La methode qui permet de changer le type d'affichage du contenu de la RAM(binaire, decimal,  ou hexa)
    * @param inType : entier codant le type
    */
  public void setAffichage( int inType ) {
    mDataModel.changeTypeAffichage( inType );
    mRamTable.revalidate();
    mRamTable.repaint();
  }

  /**
    * La methode qui permet de changer des valeurs dans la memoire
    */
  public void setAllowed() {
    mDataModel.setColonneEditable( 1 );
    mIsBeginer = false;
  }

  /**
    * La methode qui interdit de changer des valeurs dans la memoire
    */
  public void setForbidden()
  {
    mRamTable.editCellAt(0, 0 );
    mRamTable.clearSelection();
    mDataModel.setColonneEditable(-1 );
    mIsBeginer = true;
  }

  /**
    * La methode d'acces a un des combobox
    * @return la JComboBox citee
    */
  public  JComboBox getComboRam(){ return mRamViewCombo; }

  /**
    * La methode d'acces a un des combobox
    * @return la JComboBox citee
    */
  public  JComboBox getComboContenu(){ return mViewTypeCombo; }

  /**
    * La methode qui cre la fenetre de choix de la RAM
    * Elle est appele par RamControl lorsque l'utilisateur choisit "Au choix"
    * dans le JComboBox ComboVueRAM
    * @see Controlers.RamControl
    */
  public void showChoice() {
    mChoiceFrame = new RamChoiceFrame( this );
    mControl.createListenerForChoice();
  }

  /**
    * La methode d'acces a  la fenetre de choix de la RAM
    */
  public RamChoiceFrame getRamChoiceFrame(){ return mChoiceFrame; }
}
