/**
 * Le modele de table pour la fenetre des registres
 *
 * @version 1.0 29/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.IReg;

import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;

import Interface.IReg.UIReg;
import Utilitaires.*;

public class RegDataModel extends AbstractTableModel
{

    /**
     *La fenetre contenant le tableau des registres
     */
    private UIReg ui;

    /**
     *Le contenu du tableau des registres
     */
    private Vector content;

    /**
     *Le contenu du registre RCC
     */
    private Vector rcc;

    /**
     *Une variable contenant le nom des colonnes
     */
    private String[] columnNames;

   /**
     * le numero de la colonne editable
     */
    private int colonneEditable=1;

   /**
     * le type d'affichage du contenu: binaire(defaut) decimal ou hexadecimal
     */
    private int typeAffichage=PixConstants.BINAIRE;

    /**
     *Le constructeur
     * @param uiRam : la fenetre contenant la JTable ayant cette classe pour modele
     * @param column : les noms des colonnes du tableau
     * @param contenu: le contenu du tableau
     */
    public RegDataModel(UIReg uiReg,Vector contenu,Vector rccVect,String[] column)
    {
  ui=uiReg;
  content=contenu;
  rcc=rccVect;
  columnNames=column;

    }

    /**
     *Une methode a redefinir obligatoirement mais inutile ici
     * @see javax.swing.table.AbstractTableModel#getColumnCount()
     */
    public int getColumnCount() { return columnNames.length; }

    /**
     *Une methode a redefinir obligatoirement mais inutile ici
     * @see javax.swing.table.AbstractTableModel#getRowCount()
     */
    public int getRowCount() { return (content.size()+1 );
}

    /**
     * Une methode a redefinir obligatoirement.
     * Elle remplit la table a chaque reaffichage de celle ci
     * @see javax.swing.table.AbstractTableModel#getValueAt(int row, int col)
     */
    public Object getValueAt(int row, int col)
    {
  if (col==0)
      {
    if(row!=8){  return (new String(" R"+row) );
}
    else return (new String(" Rcc") );
      }
  else   //col==1
      {
    switch (typeAffichage)
        {
        case (PixConstants.DECIMAL):
      if(row!=8){return (""+(((Integer)content.elementAt(row))).toString() );
}
      else  return ("C:"+rcc.elementAt(0)+" Z:"+rcc.elementAt(1)+" N:"+rcc.elementAt(2)+" V:"+rcc.elementAt(3) );
        case (PixConstants.BINAIRE):
      if(row!=8) {return Utils.signedIntToBin(((Integer)content.elementAt(row)).intValue() );
}
      else  return ("C:"+rcc.elementAt(0)+" Z:"+rcc.elementAt(1)+" N:"+rcc.elementAt(2)+" V:"+rcc.elementAt(3) );
        case (PixConstants.HEXADECIMAL):
      if(row!=8) {return Utils.signedIntToHex(((Integer)content.elementAt(row)).intValue() );
}
      else return ("C:"+rcc.elementAt(0)+" Z:"+rcc.elementAt(1)+" N:"+rcc.elementAt(2)+" V:"+rcc.elementAt(3) );
        default:return "Erreur impossible";
        }

      }
    }

    /**
     * Une methode a redefinir obligatoirement.
     * Elle renvoie le nom de la colonne
     * @see javax.swing.table.AbstractTableModel#getColumnName(int column)
     */
    public String getColumnName(int column) {return columnNames[column];}

    /**
     * Une methode a redefinir obligatoirement.
     * Elle determine quelle partie du tableau est editable
     * @see javax.swing.table.AbstractTableModel#isCellEditable(int row, int col)
     */
    public boolean isCellEditable(int row, int col) {return ((col==colonneEditable) && (row!=8) &&(row!=0) );
}

    /**
     * Une methode a redefinir obligatoirement.
     * Elle permet de rentrer manuellemnt une valeur dans la table
     * @see javax.swing.table.AbstractTableModel#setValueAt(Object aValue, int row, int col)
     */
    public void setValueAt(Object aValue, int row, int column)
    {
  try{
      Integer valeur=new Integer(0 );
      if (typeAffichage==PixConstants.DECIMAL)
    {
        valeur=new Integer((String)aValue );
        if (valeur.intValue()<-32768 || valeur.intValue()>32767 )
      {throw new NumberFormatException("" );
}
    }
      else if (typeAffichage==PixConstants.BINAIRE)
    {
        if (((String)aValue).length()>16)
      {throw new NumberFormatException("" );
}
        valeur=new Integer(Utils.signedBinToInt(Utils.unsignedExtension((String)aValue,16)) );
    }
      else if (typeAffichage==PixConstants.HEXADECIMAL)
    {
        if (((String)aValue).length()>4)
      {throw new NumberFormatException("" );
}
        valeur=new Integer(Utils.signedHexToInt(Utils.setToNuHexChars((String)aValue,4),16) );
    }
      content.setElementAt(valeur,row );
      ui.setRegFromUI(valeur,row );
  }catch (NumberFormatException e){}
    }



    /**
     * La methode qui change le type d'affichage du contenu des registres
     * @param type : le nouveau type d'affichage
     */
    public void changeTypeAffichage(int type)
    {
  typeAffichage=type;

    }



    /**
     * La methode qui determine quelle colonne est editable
     * @param x : la colonne editable, -1 si aucune n'est editable
     */
    public void setColonneEditable(int x) { colonneEditable=x;}

    /**
     * La methode d'acces au type d'affichage
     * @return binaire,decimel ou hexa
     */
    public int getDisplayType() { return typeAffichage;}
}
