/**
* La partie de l'interface  gerant la fenetre d'affichage des registres
* Elle contient la table o est stockee le contenu des registres
*
* @version 1.0 29/03/00
* @author Paul Ladeveze
* @author Frederik Sackx
*/
package Interface.IReg;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.Vector;
import javax.swing.border.*;
import java.awt.event.*;
import javax.accessibility.*;

import Controlers.RegControl;
import Utilitaires.*;

public class UIReg extends JPanel

{
  /**
   * Le modele du tableau des registres
   */
  private RegDataModel  mDataModel;
  /**
   * les registres de 0 a 7
   */
  private Vector        mRegs;
  /**
   * le register RCC
   */
  private Vector        mRcc;
  /**
   * Le controleur des registres
   */
  private RegControl    mControl;
  /**
   * La variable determinant si le mode est novice (ou expert)
   */
  boolean               mIsNovice = true;
  
  private JScrollPane   mScrollPane;
  private JTable        mRegTable;
  private String[]      colTab= { "REGISTRE", "CONTENU" };
  
  /**
   * le constructeur
   */
  public UIReg()
  {
    super();
    
    GridBagLayout grille = new GridBagLayout();
    GridBagConstraints regles = new GridBagConstraints();
    
    setLayout( grille );

    regles.fill = GridBagConstraints.BOTH;
    regles.anchor = GridBagConstraints.CENTER;
    
    Insets interior = new Insets(1,1,1,1 );
    
    mScrollPane = new JScrollPane();
    mScrollPane.setPreferredSize( getPreferredSize() );

    
    GridUtils.buildConstraints( regles, interior, 0, 0, 0, 0, 1, 1, 100, 100 );

    grille.setConstraints( mScrollPane, regles );

    add( mScrollPane );

  }

  /**
   * La methode qui definit le controleur
   * @ param rc : le controleur
   */
  public void setControler( RegControl rc ) { mControl = rc; }
  
  /**
   * La methode qui cree le tableau et qui avait besoin au prealable du controleur
   */
  public void setData()
  {
    mControl.setRegInUI();
    mControl.setRccInUI();
    mDataModel = new RegDataModel( this, mRegs, mRcc, colTab );

    mRegTable = new JTable( mDataModel );
    mRegTable.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn registreColumn = mRegTable.getColumn( "REGISTRE" );
    registreColumn.setPreferredWidth( 100 );
    TableColumn contentColumn = mRegTable.getColumn( "CONTENU" );
    contentColumn.setPreferredWidth( 400 );
    
    mRegTable.setPreferredScrollableViewportSize( mScrollPane.getPreferredSize() );
    mScrollPane.getViewport().add( mRegTable );
    setForbidden();
  }
  
  /**
   * La methode qui permet de remplacer le contenu du modele de registres
   * @param val : le nouveau contenu
   * @param col : le numero de registre concerne
   * @see Controlers.RegControl#setRegFromUI(String ,int )
   */
  public void setRegFromUI( Integer val, int col )
  {
    mRegs.setElementAt( val, col );
    mControl.setRegFromUI( val, col );
  }
  
  
  /**
   * La methode d'acces au modele du tableau
   * @return le modele de JTable
   */
  public RegDataModel getRegDataModel() { return mDataModel; }
  
  /**
   * La methode qui permet de remplir les registres a l'affichage a partir du modele
   * @param vect : le contenu des registres(pris dans le modele sous forme d'un Vector)
   */
  public void setReg( Vector vect )
  {
    mRegs=(Vector)vect.clone();
  }
  
  /**
   * La methode qui construit les nouveaux registres
   */
  public void refreshReg()
  {
    int typeS = mDataModel.getDisplayType();
    mDataModel = new RegDataModel(this,mRegs,mRcc,colTab );
    mRegTable = new JTable(mDataModel );
    mRegTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN );
    TableColumn registreColumn = mRegTable.getColumn("REGISTRE" );
    registreColumn.setPreferredWidth(100 );
    TableColumn contentColumn = mRegTable.getColumn("CONTENU" );
    contentColumn.setPreferredWidth(400 );
    
    mRegTable.setPreferredScrollableViewportSize(mScrollPane.getPreferredSize() );
    mScrollPane.getViewport().add(mRegTable );
    setAffichage(typeS );
    setForbidden();
  }
  
  /**
   * La methode qui permet de remplir le registre RCC a l'affichage a partir du modele
   * @param vect : le contenu du registre RCC(pris dans le modele sous forme d'un Vector)
   */
  public void setRcc(Vector vect)
  {
  mRcc=(Vector)vect.clone();
  }
  
  
  /**
   * La methode qui permet de remplacer le contenu d'un registres dans l'affichage des registres
   * @param val : la nouvelle valeur
   * @param col : le numero de registre concerne
   */
  public void setRegAt( Integer val, int col )
  {
    mRegs.setElementAt( val,col );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
  
  /**
   * La methode qui permet de remplacer le contenu du registre mRcc pour le flag C
   * @param val : la nouvelle valeur
   */
  public void setRccC( int val )
  {
    mRcc.setElementAt( new Integer(val), 0 );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
  
  /**
   * La methode qui permet de remplacer le contenu du registre mRcc pour le flag Z
   * @param val : la nouvelle valeur
   */
  public void setRccZ( int val )
  {
    mRcc.setElementAt(new Integer(val),1 );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
  
  /**
   * La methode qui permet de remplacer le contenu du registre mRcc pour le flag N
   * @param val : la nouvelle valeur
   */
  public void setRccN( int val )
  {
    mRcc.setElementAt(new Integer(val),2 );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
  
  /**
   * La methode qui permet de remplacer le contenu du registre mRcc pour le flag V
   * @param val : la nouvelle valeur
   */
  public void setRccV(int val)
  {
    mRcc.setElementAt(new Integer(val),3 );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
  
  /**
   * La methode qui permet de changer des valeurs dans les registres
   */
  public void setAllowed()
  {
    mDataModel.setColonneEditable(1 );
    mIsNovice = false;
  }
  
  /**
   * La methode qui interdit de changer des valeurs dans les registres
   */
  public void setForbidden()
  {
    mRegTable.editCellAt( 0,0 );
    mRegTable.clearSelection();
    mDataModel.setColonneEditable( -1 );
    mIsNovice = true;
  }
  
  /**
   * La methode d'acces au  tableau des registres
   * @return le tableau des registres
   */
  public  Vector getReg() { return mRegs; }
  
  /**
   * La methode d'acces a la table des registres
   * @return la table des registres
   */
  public JTable getTable() { return mRegTable; }
  
  /**
   * La methode qui permet de changer le type d'affichage du contenu des registres(binaire,decimal, ou hexa)
   * @param type : entier codant le type
   */
  public void setAffichage( int type )
  {
    mDataModel.changeTypeAffichage( type );
    mRegTable.revalidate();
    mRegTable.repaint();
  }
}
