/**
 * Le menu contenu dans la fenetre principale de l'application
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Interface.UI;

import javax.swing.*;
import java.awt.*;

import Controlers.UIControl;
import Interface.UI.UI;


public class Menu extends JMenuBar
{
  private JMenu help = new JMenu( "Aide" );
  private JMenu lFile = new JMenu( "Fichier" );
  private JMenu parametres = new JMenu( "Paramtres", true );
  private JMenu resolution = new JMenu( "Rsolution", true );
  private JMenu mode = new JMenu( "Mode",true );
  private JMenuItem quitter = new JMenuItem( "Quitter" );
  private JMenuItem ouvrir = new JMenuItem( "Charger" );
  private JMenuItem sauver = new JMenuItem( "Sauver" );
  
  private JMenuItem r640x480 = new JMenuItem( "640*480" );
  private JMenuItem r800x600 = new JMenuItem( "800*600" );
  private JMenuItem r1024x768 = new JMenuItem( "1024*768" );
  private JMenuItem demo = new JMenuItem( "Demonstration" );
  private JMenuItem novice = new JMenuItem( "Novice" );
  private JMenuItem expert = new JMenuItem( "Expert" );
  
  private JMenuItem aide = new JMenuItem( "Sommaire" );
  private JMenuItem about = new JMenuItem( "A propos" );
  
  private JCheckBoxMenuItem anim = new JCheckBoxMenuItem( "Animer la lecture d'instruction", true );

  /**
   * le controleur de la fenetre principale
   */
  private UIControl mControl;
  
  /**
   * le constructeur
   */
  public Menu()
  {
    super();
    resolution.add( r640x480 );
    resolution.add( r800x600 );
    resolution.add( r1024x768 );
    mode.add( demo );
    mode.add( novice );
    mode.add( expert );
    lFile.add( ouvrir );
    lFile.add( sauver );
    lFile.add( quitter );
    parametres.add( resolution );
    parametres.add( mode );
    anim.setFont( new Font( "DIALOG",Font.PLAIN,14) );
    parametres.add( anim );
    help.add( aide );
    help.add( about );
    add( lFile );
    add( parametres );
    add( help );
    anim.setEnabled( false );
  }
  
  /**
   * Change la police et les icones pour le mode 1024*768
   */
  public void setBigIcons()
  {
    Font fb = new Font( "DIALOG",Font.PLAIN,14 );
    help.setFont( fb );
    lFile.setFont( fb );
    parametres.setFont( fb );
    resolution.setFont( fb );
    ouvrir.setFont( fb );
    sauver.setFont( fb );
    quitter.setFont( fb );
    anim.setFont( fb );
    mode.setFont( fb );
    demo.setFont( fb );
    novice.setFont( fb );
    expert.setFont( fb );
    r640x480.setFont( fb );
    r800x600.setFont( fb );
    r1024x768.setFont( fb );
    aide.setFont( fb );
    about.setFont( fb );
  }
  
  
  /**
   * Change la police et les icones pour le mode 800*600
   */
  public void setMedIcons()
  {
    Font fb = new Font( "DIALOG",Font.PLAIN,12 );
    help.setFont( fb );
    lFile.setFont( fb );
    parametres.setFont( fb );
    resolution.setFont( fb );
    ouvrir.setFont( fb );
    sauver.setFont( fb );
    quitter.setFont( fb );
    anim.setFont( fb );
    mode.setFont( fb );
    demo.setFont( fb );
    novice.setFont( fb );
    expert.setFont( fb );
    r640x480.setFont( fb );
    r800x600.setFont( fb );
    r1024x768.setFont( fb );
    aide.setFont( fb );
    about.setFont( fb );
  }
  
  
  /**
   * Change la police et les icones pour le mode 640*480
   */
  public void setMiniIcons()
  {
    Font fp = new Font( "DIALOG",Font.PLAIN,10 );
    help.setFont( fp );
    lFile.setFont( fp );
    parametres.setFont( fp );
    resolution.setFont( fp );
    mode.setFont( fp );
    anim.setFont( fp );
    demo.setFont( fp );
    novice.setFont( fp );
    expert.setFont( fp );
    ouvrir.setFont( fp );
    sauver.setFont( fp );
    quitter.setFont( fp );
    r640x480.setFont( fp );
    r800x600.setFont( fp );
    r1024x768.setFont( fp );
    aide.setFont( fp );
    about.setFont( fp );
  }
  
  
  /**
   * La methode qui definit le controleur
   * @param inControl : le controleur d'UI
   */
  public void setControler( UIControl inControl )
  {
    mControl = inControl;
    ouvrir.addActionListener( inControl );
    sauver.addActionListener( inControl );
    quitter.addActionListener( inControl );
    aide.addActionListener( inControl );
    about.addActionListener( inControl );
    r640x480.addActionListener( inControl );
    r800x600.addActionListener( inControl );
    r1024x768.addActionListener( inControl );
    demo.addActionListener( inControl );
    novice.addActionListener( inControl );
    expert.addActionListener( inControl );
    anim.addActionListener( inControl );
  }
  
  /**
   * La methode d'acces au menu lFile->ouvrir
   * @return la composante du menu
   */
  public JMenuItem getOuvrir() { return ouvrir; }
  /**
   * La methode d'acces au menu lFile->sauver
   * @return la composante du menu
   */
  public JMenuItem getSauver() { return sauver; }
  
  /**
   * La methode d'acces au menu lFile->quitter
   * @return la composante du menu
   */
  public JMenuItem getQuitter()
  {
  return quitter;
  }
  /**
   * La methode d'acces au menu parametres->resolution->640x480
   * @return la composante du menu
   */
  public JMenuItem getR640x480()
  {
  return r640x480;
  }
  
  /**
   * La methode d'acces au menu parametres->resolution->800x600
   * @return la composante du menu
   */
  public JMenuItem getR800x600()
  {
  return r800x600;
  }
  
  /**
   * La methode d'acces au menu parametres->resolution->1024x768
   * @return la composante du menu
   */
  public JMenuItem getR1024x768()
  {
  return r1024x768;
  }
  
  /**
   * La methode d'acces au menu mode->novice
   * @return la composante du menu
   */
  public  JMenuItem getNovice()
  {
  return novice;
  }
  
  
  /**
   * La methode d'acces au menu mode->expert
   * @return la composante du menu
   */
  public  JMenuItem getExpert()
  {
  return expert;
  }
  
  
  /**
   * La methode d'acces au menu mode->demo
   * @return la composante du menu
   */
  public  JMenuItem getDemo()
  {
  return demo;
  }
  
  
  /**
   * La methode d'acces au menu aide
   * @return la composante du menu
   */
  public JMenuItem getAide()
  {
  return aide;
  }
  
  
  /**
   * La methode d'acces au menu
   * @return la composante du menu
   */
  public JMenuItem getParametres()
  {
  return parametres;
  }
  
  
  /**
   * La methode d'acces au menu
   * @return la composante du menu
   */
  public JMenuItem getHelp()
  {
  return help;
  }
  
  
  /**
   * La methode d'acces au menu
   * @return la composante du menu
   */
  public JMenuItem getAnim()
  {
  return anim;
  }
  
  /**
   * La methode d'acces a about
   * @return about
   */
  public JMenuItem getAbout()
  {
  return about;
  }
  
}
