/**
 * La partie de l'interface  gerant les differents modeles: RAM,edition,architecture.
 * Elle represente la fenetre principale.
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */

package Interface.UI;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import Interface.IEdit.UIEdit;
import Interface.IRam.UIRam;
import Interface.IReg.UIReg;
import Interface.IArchi.UIArchi;
import Interface.ICmd.UICmd;
import Utilitaires.*;

public class UI extends JFrame implements WindowListener
{
  /**
   * Le paneau contenant la RAM
   */
  private  UIRam interRam =new UIRam();

  /**
   * Le paneau contenant les regitres
   */
  private  UIReg interReg =new UIReg();

  /**
   * Le paneau contenant le schema d'architecture
   */
  private  UIArchi interArchi =new UIArchi();

  /**
   * Le paneau contenant la barre des commandes
   */
  private  UICmd interCmd =new UICmd();

  /**
   * Le paneau contenant la boite d'edition
   */
  private  UIEdit interEdit =new UIEdit();

  /**
   * Le menu deroulant
   */
  private Menu menu = new Menu();

  /**
   * Le constructeur
   */
  public UI()
  {
    super("PixArchi : Mode Novice" );
    setEnabled(false );
    setResizable(false );
    setStartResolution();
    setJMenuBar(menu );
    addWindowListener( this );

    try {
      UIManager.setLookAndFeel( "javax.swing.plaf.metal.MetalLookAndFeel" );

      SwingUtilities.updateComponentTreeUI( this );

    } catch (Exception e)
    { System.err.println("Impossible d'appliquer l'apparence : " + e );
 }

    GridBagLayout grille = new GridBagLayout();
    GridBagConstraints regles = new GridBagConstraints();

    getContentPane().setLayout( grille );

    regles.fill = GridBagConstraints.BOTH;
    regles.anchor = GridBagConstraints.CENTER;
    Insets interior = new Insets(1,1,1,1 );

    GridUtils.buildConstraints( regles, interior, 0, 0, 0, 0, 1, 3, 50, 100 );

    interArchi.setBorder( new SoftBevelBorder( SoftBevelBorder.RAISED ) );

    grille.setConstraints( interArchi, regles );

    getContentPane().add( interArchi );


    GridUtils.buildConstraints( regles, interior, 0, 0, 1, 0, 1, 1, 50, 45 );

    interRam.setBorder( new SoftBevelBorder( SoftBevelBorder.RAISED ) );

    grille.setConstraints(interRam,regles );
    //interRam.getComboRam().setEnabled(false );
    //interRam.getComboContenu().setEnabled(false );
    getContentPane().add(interRam );

    GridUtils.buildConstraints(regles,interior,0,0,1,1,1,1,50,25 );
    interReg.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED) );
    grille.setConstraints(interReg,regles );
    getContentPane().add(interReg );

    GridUtils.buildConstraints(regles,interior,0,0,1,2,1,1,50,30 );
    interEdit.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED) );
    grille.setConstraints(interEdit,regles );
    getContentPane().add(interEdit );

    GridUtils.buildConstraints(regles,interior,0,0,0,3,2,1,100,6 );
    interCmd.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED) );
    grille.setConstraints(interCmd,regles );
    getContentPane().add(interCmd );

    //setVisible( true );
    setEnabled( true );
  }

//  public addInterBox( GridBagLayout inGrid, GridBagConstraints inRules, Insets inInsets,  )

/**
* La methode qui change la resolution de l'interface toute entiere
* @param taillex : nouvelle hauteur
* @param tailley : nouvelle largeur
*/
public void setResolution(int taillex, int tailley)
{
  setEnabled(false );
  setSize(taillex,tailley );
  interArchi.setImageSize(taillex+"-"+tailley );
  setIcons(taillex,tailley );
  setEnabled( true );
}


/**
* La methode qui change les icones et les polices de l'interface suivant la rsolution
* @param taillex : nouvelle hauteur
* @param tailley : nouvelle largeur
*/
public void setIcons(int taillex,int tailley)
{
if (taillex==640 && tailley==480)
{
interCmd.setMiniIcons();
interEdit.setMiniIcons();
menu.setMiniIcons();
interArchi.setMiniIcons();
interRam.setMiniIcons();
}
else if (taillex==800 && tailley==600)
{
interCmd.setMedIcons();
interEdit.setMedIcons();
menu.setMedIcons();
interArchi.setMedIcons();
interRam.setMedIcons();
}
else if (taillex>=1024 && tailley>=768)
{
interCmd.setBigIcons();
interEdit.setBigIcons();
menu.setBigIcons();
interArchi.setBigIcons();
interRam.setBigIcons();
}
}

/**
* La methode qui change la resolution de l'interface toute entiere au DEMARRAGE
*/
public void setStartResolution()
{
setEnabled(false );
boolean fixed=false;
Dimension dim=getToolkit().getScreenSize();
int hauteur=(int)dim.getHeight();
int largeur=(int)dim.getWidth();
if ((largeur==640 && hauteur==480)&& (!fixed))
{
setSize(largeur,hauteur );
interArchi.setImageSize(largeur+"-"+hauteur );
setIcons(640,480 );
fixed=true;
}
if ((largeur==800 && hauteur==600)&& (!fixed))
{
setSize(640,480 );
interArchi.setImageSize(640+"-"+480 );
setIcons(640,480 );
fixed=true;
}
if ((largeur==1024 && hauteur==768)&& (!fixed))
{
setSize(800,600 );
interArchi.setImageSize(800+"-"+600 );
setIcons(800,600 );
fixed=true;
}
if ((largeur>1024 && hauteur>768)&& (!fixed))
{
setSize(1024,768 );
interArchi.setImageSize(1024+"-"+768 );
setIcons(1024,768 );
fixed=true;
}
if (!fixed)
{
setSize(640,480 );
setIcons(640,480 );
interArchi.setImageSize(640+"-"+480 );
}

// on centre la fenetre
setLocation(dim.width/2 -this.getWidth()/2,dim.height/2 -this.getHeight()/2 );
setEnabled( true );
}

/**
* La methode qui dit si la lecture d'une instruction doit tre anime ou non
*/
public void setAnimLect(boolean b)
{
interArchi.setAnimLect(b );
}

/**
* La methode d'acces au paneau de RAM
* @return la partie de l'interface montrant la RAM
*/
public UIRam getUIRam()
{
return interRam;
}

/**
* La methode d'acces au paneau des registres
* @return la partie de l'interface montrant les registres
*/
public UIReg getUIReg()
{
return interReg;
}

/**
* La methode d'acces au paneau de commandes
* @return la partie de l'interface servant de barre de commandes
*/
public UICmd getUICmd()
{
return interCmd;
}

/**
* La methode d'acces au paneau d'edition
* @return la partie de l'interface montrant les instructions assembleurs sur lesquelles on travaille
*/ public UIEdit getUIEdit()
{
return interEdit;
}

/**
* La methode d'acces au paneau du schema d'architecture
* @return la partie de l'interface montrant le schema et l'etat d'execution
*/ public UIArchi getUIArchi()
{
return interArchi;
}

/**
* La methode d'acces au menu deroulant
* @return le menu deroulant
*/
public Menu getMenu()
{
return menu;
}



  /*Les mthodes a redfinir pour l'interface WindowListener*/
  /*inutile a commenter !!!!!*/
  public void windowActivated(WindowEvent e) {}
  public void windowClosed(WindowEvent evt) { System.exit(0 );
 }
  public void windowClosing(WindowEvent e) { System.exit(0 );
 }
  public void windowDeactivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
}
