package Model;

import java.util.Vector;

import javax.swing.Timer;

import Controlers.*;
import Utilitaires.*;

/**
 * le moteur gerant les differents modeles: RAM,edition,architecture,commandes
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 * @authoe Yves Lhuillier
 */
public class Engine
{
  /** Le controleur du timer */
  private EngineControl mEngineControl;
  /** le modele de RAM */
  private MRam          mRamEngine = new MRam();
  /** le modele des registres */
  private MReg          mRegEngine = new MReg();
  /** le modele d'editeur */
  private MEdit         mEditEngine = new MEdit();
  /** le modele de l'architecture */
  private MArchi        mArchiEngine = new MArchi( this );
  /** le modele des boutond de commande */
  private MCmd          mCmdEngine = new MCmd();
  /** Variable qui donne l'tat de l'excution : PLAY,PAUSE,STOP */
  private int           mState;
  /** L'instruction courante */
  Quintuplet            mInstruction;
  /** Variables qui donnent la position en n d'instruction de l'execution */
  private int           mPosition;
  private int           mPositionTmp;
  private Vector        mPosHistory;
  /** Variables qui donnent le nombre de fois que l'on est pass par chaque instruction */
  private Vector        mPositionCpt;
  private Vector        mPositionCptTmp;
  /** Variables qui donnent le pas de l'execution */
  private int           mStep;
  private int           mStepTmp;
  /** Variable qui contient les textes des pas d'excution */
  private Vector        mStepInfo = new Vector();
  /** Variable qui contient les informations d'animation */
  private Vector        animInfo = new Vector();
  /** Le timer qui gere le delai d'execution */
  private Timer         mDelai;
  /** Variable qui dit s'il faut animer ou non */
  private boolean       mAnimate = false;
  /** Variable qui dit s'il faut animer ou non */
  private boolean       mNotAnimate = false;
  /** Variable qui dit s'il faut animer la lecture d'instruction ou non */
  private boolean       mAnimLect = true;
  /** Variable qui permet de savoir si le dernier saut conditionnel ou non a t effectu */
  private boolean       mJump = true;

  /** le constructeur  */
  public Engine()
  {
    mState = PixConstants.STOP;
    mPosition = -1;
    mPositionTmp = -1;
    mStep = -1;
  }

  /** La methode qui ajoute le controleur d'evenements au timer */
  public void setData()
  {
    mDelai = new Timer( 2000, mEngineControl );
    mDelai.setInitialDelay( 2000 );
    mDelai.setRepeats( false );
  }

  /**
   * Methode de soumission du controleur du timer
   * @param inEngineControl : le controleur a rcuprer
   */
  public void setControler( EngineControl inEngineControl ) { mEngineControl = inEngineControl; }

  /**
   * La methode d'acces au modele de RAM
   * @return le modele de RAM
   */
  public MRam getMRam() { return mRamEngine; }

  /**
   * La methode d'acces au modele des registres
   * @return le modele des registres
   */
  public MReg getMReg() { return mRegEngine; }

  /**
   * La methode d'acces au modele d'edition
   * @return le modele d'edition
   */
  public MEdit getMEdit() { return mEditEngine; }

  /**
   * La methode d'acces au modele d'architecture
   * @return le modele d'architecture
   */
  public MArchi getMArchi() { return mArchiEngine; }

  /**
   * La methode d'acces au modele des commandes
   * @return le modele des commandes
   */
  public MCmd getMCmd() { return mCmdEngine; }

  /**
   * La methode d'acces au timer
   * @return Le timer
   */
  public Timer getTimer() { return mDelai; }


  /**
   * La methode qui change le delai du timer
   * @param t : le nouveau dlai
   */
  public void setTime( int t )
  {
    mDelai.setDelay( t * 1000 );
    mDelai.setInitialDelay( t * 1000 );
  }


  /**
   * La methode qui initialise les compteurs d'instruction
   */
  public void initPositionsCpt()
  {
    mPositionCpt = new Vector();
    int taille = mEditEngine.getProgram().size();
    for( int t = 0; t < taille + 1; t++ )
      mPositionCpt.addElement( new Integer( 0 ) );
  }


  /**
   * La methode qui incremente un compteur d'instruction
   * @param lIndex : l'indice de l'instruction
   */
  public void incPositionsCptAt( int lIndex )
  {
    int nb = ((Integer)mPositionCpt.elementAt( lIndex )).intValue();
    mPositionCpt.setElementAt( new Integer( nb+1 ), lIndex );
  }


  /**
   * La methode qui decremente un compteur d'instruction
   * @param lIndex : l'indice de l'instruction
   */
  public void decPositionsCptAt( int lIndex )
  {
    int nb = ((Integer)mPositionCpt.elementAt( lIndex )).intValue();
    mPositionCpt.setElementAt( new Integer( nb-1 ), lIndex );
  }


  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur play
   */
  public void play()
  {
    try {
      mAnimate = true;
      mEditEngine.setEditEnabled( false );

      if ( mState == PixConstants.STOP )
      {
        mEditEngine.unEdit();
        mRamEngine.initAlter();
        mEditEngine.launchAnalyse();
        mArchiEngine.setPC(1 );
        assemble();
        initPositionsCpt();
        mPosHistory = new Vector();
        mRamEngine.saveRam();
        mRegEngine.saveReg();

        mState = PixConstants.PLAY;
        mCmdEngine.setStepsEnabled( false );

        execute();
      }
      else if (mState==PixConstants.PAUSE)
      {
        String s = mArchiEngine.getMsg();
        s=s.substring(0,s.length()-7 );
        mArchiEngine.setMsg(s );

        mState=PixConstants.PLAY;
        mCmdEngine.setStepsEnabled(false );
        execute();
      }
    }
    catch (NumberFormatException e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.setEditEnabled( true );
      mEditEngine.showErrorFormat( "" + e.getMessage() );
    }
    catch (Exception e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.setEditEnabled( true );
      mEditEngine.showErrorSyntax( e.getMessage() );
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur pause
   */
  public void pause()
  {
    mArchiEngine.stopAnim();
    if (mState==PixConstants.PLAY)
    {
      mCmdEngine.setStepsEnabled( true );
      mState = PixConstants.PAUSE;
    }
    else if (mState==PixConstants.STOP)
    {
      mCmdEngine.setStepsEnabled( true );
      mCmdEngine.setStopSelected();
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur stop
   */
  public void stop()
  {
    mArchiEngine.stopAnim();
    if ( mState == PixConstants.PLAY || mState == PixConstants.PAUSE )
    {
      mCmdEngine.setStopSelected();
      mState = PixConstants.STOP;
      mArchiEngine.resetColor();
      mCmdEngine.setStepsEnabled( true );
      mPosition = -1;
      mPositionTmp = -1;
      mStep = -1;
      mArchiEngine.setMsg("Pas d'excution en cours" );
      mArchiEngine.setPC(1 );
      mArchiEngine.updateArchi();
      mEditEngine.setEditEnabled( true );
      mStepInfo=new Vector();
      mEditEngine.unEdit();
      mArchiEngine.clear();
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur backStep
   */
  public void backStep()
  {
    mAnimate = false;
    mArchiEngine.stopAnim();
    if ( mState == PixConstants.STOP ) {}
    else if ( mState == PixConstants.PAUSE && !( mPosition <= 1 && mStep == 0 ))
    {
      mCmdEngine.setStepsEnabled( false );

      int posTemp, pasTemp;
      if ( mStep == 0 )
      {
        mPosHistory.removeElementAt( mPosHistory.size() - 1 );

        posTemp = ((Integer)mPosHistory.elementAt( mPosHistory.size() - 1 )).intValue();
        decPositionsCptAt( mPosition );
        mInstruction = (Quintuplet)mEditEngine.getProgram().elementAt( posTemp - 1 );
        mStepInfo = StepInfo.getStepInfo( mInstruction );
        pasTemp = mStepInfo.size();
      }
      else
      {
        posTemp = mPosition;
        pasTemp = mStep - 1;
      }
      mCmdEngine.setPauseSelected();
      executeFromBeginTo( posTemp, pasTemp );

      if ( mStep == 0 )
        mArchiEngine.setPC( mPosition );

      else
        mArchiEngine.setPC( mPosition + 1 );

      mArchiEngine.updateArchi();
      mAnimate = true;
      if ( mStep == 0 )
        execMInstruction();
      else
        execPasMInstruction();
      mAnimate = false;
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur back
   */
  public void back()
  {
    mAnimate = false;
    mArchiEngine.stopAnim();
    if ( mState == PixConstants.STOP ) {}
    else if ( mState == PixConstants.PAUSE )
    {
      mCmdEngine.setStepsEnabled( false );

      mCmdEngine.setPauseSelected();
      if ( mStep == 0 && mPosition > 1)
      {
        mPosHistory.removeElementAt( mPosHistory.size() - 1 );

        int posTemp = ((Integer)mPosHistory.elementAt( mPosHistory.size() - 1 )).intValue();
        decPositionsCptAt( mPosition );
        executeFromBeginTo( posTemp, 0 );
      }
      else if (mStep!=0 && mPosition>=1)
      {
        executeFromBeginTo( mPosition, 0 );
      }
      mArchiEngine.setPC( mPosition );

      mArchiEngine.updateArchi();
      mAnimate=true;
      execMInstruction();
      mAnimate=false;
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur nextstep
   */
  public void nextStep()
  {
    if (!mNotAnimate) { mAnimate=true; }
    mArchiEngine.stopAnim();
    try{
      mEditEngine.setEditEnabled( false );

      if (mState==PixConstants.STOP)
      {
        mEditEngine.unEdit();
        mEditEngine.launchAnalyse();
        mArchiEngine.setPC(1 );
        assemble();
        initPositionsCpt();
        mPosHistory=new Vector();
        mRamEngine.saveRam();
        mRegEngine.saveReg();
        mState=PixConstants.PAUSE;
        mArchiEngine.resetColor();
        mCmdEngine.setPauseSelected();
      }
      if ( mState == PixConstants.PAUSE && !((mPositionTmp + 1) > mEditEngine.getProgram().size() && mStep == mStepInfo.size()) )
      {
        mCmdEngine.setPauseSelected();
        if (mStep==mStepInfo.size() || mPosition <= 0)
        {
          mStep=-1;
          mPositionTmp++;
          mPosition = mPositionTmp;
          mPosHistory.addElement( new Integer( mPosition ) );
          mArchiEngine.resetColor();
          incPositionsCptAt(mPosition );
          if (mPosition!=0) { mEditEngine.setSelected(mEditEngine.seekPosInEdit(mPosition) ); }
          mStep++;
          execMInstruction();
        }
        else
        {
          mStep++;
          execPasMInstruction();
        }
      }
      else if (mState==PixConstants.PAUSE)
      {
        if ( mInstruction.getComd().substring(0,1).equals("b") && mJump ) { mEditEngine.showErrorJump(""+(mPositionTmp+1) ); }
        stop();
      }
    }
    catch (NumberFormatException e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.setEditEnabled( true );
      mEditEngine.showErrorFormat(""+e.getMessage() );
    }
    catch (Exception e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.showErrorSyntax(""+e.getMessage() );
      mEditEngine.setEditEnabled( true );
    }
  }

  /**
   * Methode invoquee par le controleur lorsque
   * l'utilisateur appuie sur next
   */
  public void next()
  {
    mAnimate=false;
    mArchiEngine.stopAnim();
    try {
      mEditEngine.setEditEnabled(false );
      if (mState==PixConstants.STOP)
      {
        mEditEngine.unEdit();
        mEditEngine.launchAnalyse();
        mArchiEngine.setPC(1 );
        assemble();
        initPositionsCpt();
        mPosHistory=new Vector();
        mRamEngine.saveRam();
        mRegEngine.saveReg();
        mState=PixConstants.PAUSE;
        mCmdEngine.setPauseSelected();
        mStep=0;
        execNextMInstruction();
        mAnimate=true;
        execMInstruction();
        mAnimate=false;
      }
      else if (mState==PixConstants.PAUSE && mPositionTmp<=mEditEngine.getProgram().size())
      {
        mStep=0;
        mCmdEngine.setPauseSelected();
        execNextMInstruction();
        mAnimate=true;
        execMInstruction();
        mAnimate=false;
      }
      else if (mState==PixConstants.PAUSE)
      {
        if ( mInstruction.getComd().substring(0,1).equals("b") && mJump ) { mEditEngine.showErrorJump(""+(mPositionTmp+1) ); }
        stop();
      }
    }
    catch (NumberFormatException e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.setEditEnabled( true );
      mEditEngine.showErrorFormat(""+e.getMessage() );
    }
    catch (Exception e)
    {
      mCmdEngine.setStopSelected();
      mEditEngine.setEditEnabled( true );
      mEditEngine.showErrorSyntax(""+e.getMessage() );
    }
  }

  /**
   * Assemble le programme
   */
  private void assemble() throws Exception {
    mArchiEngine.showAllpics();
    for( int lIndex = 0; lIndex < mEditEngine.getProgram().size(); lIndex++ ) assembleLine( lIndex );
  }

  /**
   *Assemble une ligne du programme
   *@param inLine : la ligne a assembler
   */
  private void assembleLine( int inLine ) throws Exception {
    Quintuplet line = (Quintuplet)mEditEngine.getProgram().elementAt(inLine );
    String     com  = line.getComd();
    String     type;
    int        a1   = line.getArg1();
    int        a2;
    int        a3;
    int        lFormat    = line.getFormat();
    int        pos  = mEditEngine.seekPosInEdit(inLine+1 );
    String     lInstrCode = "";
    
    switch(lFormat)
    {
    case 1 :
      a2 = line.getArg2();
      if ( a1<0 || a2<0 || a2>2047 || a1>7 ) throw new NumberFormatException(""+pos );
      lInstrCode += "00" + Utils.unsignedExtension( Integer.toBinaryString(a2), 11 ) + Utils.unsignedExtension( Integer.toBinaryString(a1), 3 );
      break;
    case 2 :
      if ( a1<-1024 || a1>1023 ) throw new NumberFormatException(""+pos );
      lInstrCode += "010" + Utils.unsignedExtension( Integer.toBinaryString(a1), 11 ) + Utils.getCode(com) + "0";
      break;
    case 3 :
      if ( a1<-256 || a1>255 ) throw new NumberFormatException(""+pos );
      lInstrCode += "0110" + Utils.getCode(com)+Utils.unsignedExtension(Integer.toBinaryString(a1),9 );
      break;
    case 4 :
      a2 = line.getArg2();
      a3 = line.getArg3();
      if (a1<0 || a2<0 || a3<0 || a1>7 || a2>7 || a3>7) throw new NumberFormatException(""+pos );
      type = com.substring(0,1 );
      if( type.equals("s") || type.equals("S") )
        lInstrCode += "100" +
                Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a2),3) +
                Utils.unsignedExtension(Integer.toBinaryString(a1),3) +
                Utils.unsignedExtension(Integer.toBinaryString(a3),3 );
      else
        lInstrCode += "100" +
                Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a3),3) +
                Utils.unsignedExtension(Integer.toBinaryString(a2),3) +
                Utils.unsignedExtension(Integer.toBinaryString(a1),3 );
      break;
    case 5 :
      a2 = line.getArg2();
      a3 = line.getArg3();
      type = com.substring(0,1 );
      if (type.equals("o") || type.equals("O"))
      {
        if (a1<0 || a2<0 || a3<0 || a1>7 || a2>7 || a3>31) throw new NumberFormatException(""+pos );
        lInstrCode += "1" + Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a3),5) +
                      Utils.unsignedExtension(Integer.toBinaryString(a2),3) +
                      Utils.unsignedExtension(Integer.toBinaryString(a1),3 );
      }
      else if (type.equals("s") || type.equals("S"))
      {
        if (a1<0 || a2<-16 || a3<0 || a1>7 || a2>15 || a3>7) throw new NumberFormatException(""+pos );
        lInstrCode += "1" + Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a2),5) +
                      Utils.unsignedExtension( Integer.toBinaryString(a1), 3 ) +
                      Utils.unsignedExtension(Integer.toBinaryString(a3),3 );
      }
      else
      {
        if (a1<0 || a2<0 || a3<-16 || a1>7 || a2>7 || a3>15) throw new NumberFormatException(""+pos );
        lInstrCode += "1" + Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a3),5) +
                      Utils.unsignedExtension(Integer.toBinaryString(a2),3) +
                      Utils.unsignedExtension(Integer.toBinaryString(a1),3 );
      }
      break;
    case 6 :
      a2 = line.getArg2();
      a3 = line.getArg3();
      if (a1<0 || a2<0 || a3<0 || a1>7 || a2>7 || a3>15) throw new NumberFormatException(""+pos );
      type = com.substring(0,1 );
      lInstrCode += "1111" + Utils.getCode(com) + Utils.unsignedExtension(Integer.toBinaryString(a3),4) +
                       Utils.unsignedExtension(Integer.toBinaryString(a2),3) +
                       Utils.unsignedExtension(Integer.toBinaryString(a1),3 );
      break;
    default : break;
    }
    mRamEngine.setRamFromModel( lInstrCode,inLine );
  }

  /** Execute les instructions du programme */
  public void execute()
  {
    if ( mState == PixConstants.PLAY )
    {
      if ((mPositionTmp)>=mEditEngine.getProgram().size() && mStep==mStepInfo.size())
      {
        mCmdEngine.setStopSelected();
        if ( mInstruction.getComd().substring(0,1).equals("b") && mJump ) { mEditEngine.showErrorJump(""+(mPositionTmp+1) ); }
        stop();
      }
      else
      {
        if (mStep==mStepInfo.size())
        {
          mStep=-1;
          mPositionTmp++;
          mPosition=mPositionTmp;
          mArchiEngine.resetColor();
          mPosHistory.addElement(new Integer(mPosition) );
          incPositionsCptAt(mPosition );
        }
        if (mPosition==-1)
        {
          mPositionTmp++;
          mPosition=mPositionTmp;
          mArchiEngine.resetColor();
          mPosHistory.addElement(new Integer(mPosition) );
          incPositionsCptAt(mPosition );
        }
        mStep++;
        if (mPosition!=0) { mEditEngine.setSelected(mEditEngine.seekPosInEdit(mPosition) ); }
      }
      if ( mPosition>0 && (mPosition)<=mEditEngine.getProgram().size() )
        mEditEngine.setSelected( mEditEngine.seekPosInEdit(mPosition) );

      if (mStep<=0) { execMInstruction(); }
      else { execPasMInstruction(); }
      if ( mPosition == 0 ) continueExec();
    }
  }

  /** Methode qui poursuit l'execution lorsqu'une animation est finie */
  public void continueExec() { mDelai.restart(); }

  /** La methode qui dit si la lecture d'une instruction doit tre anime ou non */
  public void setAnimLect(boolean b) { mAnimLect=b; }

  /** Fonction permettant de comparer deux vecteurs contenant des Integer et tant de la mme taille */
  private boolean compVect(Vector cptTmp,Vector cpt)
  {
    boolean res=true;
    for ( int h=0; h < cpt.size(); h++)
      if( ((Integer)cptTmp.elementAt(h)).intValue()!=((Integer)cpt.elementAt(h)).intValue() )
      { res = false; break; }
    return res;
  }


  /** Execute les instructions du programme */
  private void executeFromBeginTo(int pos,int pasE)
  {
    mPosition=-1;
    mPositionTmp=-1;
    if (pasE==3) mArchiEngine.resetColor();
    mStep=0;
    mStepTmp=pasE;
    mPositionCptTmp=(Vector)mPositionCpt.clone();
    initPositionsCpt();
    mRamEngine.restoreRam();
    mRegEngine.restoreReg();
    mNotAnimate=true;

    //IL FAUT FAIRE L'EXECUTION DEPUIS LE DEBUT
    while( mStep!=mStepTmp || (!compVect( mPositionCptTmp, mPositionCpt )) )
    { nextStep(); }

    mNotAnimate = false;
    mCmdEngine.setStepsEnabled( true );
  }


  /** Execute l'instruction courante du programme */
  private void execNextMInstruction()
  {
    mArchiEngine.setReg1Info("-" );
    mArchiEngine.setReg2Info("-" );
    mArchiEngine.setReg3Info("-" );
    mArchiEngine.setUalInfo("\u2205" );
    mArchiEngine.updateArchi();
    mArchiEngine.afficheInstruction(mStep,mPosition,"" );
    mNotAnimate=true;

    int tempo = mPositionTmp;
    nextStep();
    while (tempo==mPosition && mStep!=0)
      nextStep();

    mNotAnimate=false;
    mArchiEngine.setPC(mPosition+1 );
    mArchiEngine.resetColor();
  }


  /** Execute l'instruction courante du programme */
  private void execMInstruction()
  {
    mArchiEngine.clear();

    mArchiEngine.setReg1Info( "-" );
    mArchiEngine.setReg2Info( "-" );
    mArchiEngine.setReg3Info( "-" );
    mArchiEngine.setUalInfo( "\u2205" );
    mArchiEngine.updateArchi();
    mArchiEngine.afficheInstruction(mStep,mPosition,"" );

    mArchiEngine.setPC(mPosition+1 );

    if(mStep==0 && mAnimate && mPosition!=0 && mAnimLect)
    {
      mArchiEngine.addAnimState( PixConstants.CPTTOBUS1ANIM );
      mArchiEngine.addAnimState( PixConstants.BUS1ANIM );
      mArchiEngine.addAnimState( PixConstants.BUS1TOMEMANIM );
      mArchiEngine.addAnimState( PixConstants.MEMTOBUS3ANIM );
      mArchiEngine.addAnimState( PixConstants.BUS3ANIM );
      mArchiEngine.addAnimState( PixConstants.BUS3TOREGIANIM );
      mArchiEngine.addAnimState( PixConstants.CPTTOCPTPLUSANIM );
      mArchiEngine.addAnimState( PixConstants.CPTPLUSTOCPTANIM );
      mArchiEngine.startAnim();
    }
    else if( mStep==0 && mAnimate && mPosition!=0 && !mAnimLect )
    {
      //mArchiEngine.updateArchi();
      continueExec();
    }

    if (mPosition>0 && (mPosition)<=mEditEngine.getProgram().size())
    {
      mEditEngine.setSelected(mEditEngine.seekPosInEdit(mPosition) );
      mInstruction=(Quintuplet)mEditEngine.getProgram().elementAt(mPosition-1 );
      mStepInfo=StepInfo.getStepInfo( mInstruction );
    }
  }

  /** Execute l'instruction courante du programme */
  private void execPasMInstruction()
  {
    if (mPosition!=0) { mEditEngine.setSelected(mEditEngine.seekPosInEdit(mPosition) ); }
    if (mStep==1)
    {
      if( mAnimate )
      {
        mArchiEngine.clear();
        mArchiEngine.updateArchi();
      }
    }
    mArchiEngine.afficheInstruction(mStep,mPosition,(String)mStepInfo.elementAt(mStep-1) );
    switch( mInstruction.getFormat())
    {
    case 1 :
      if( mInstruction.getComd().equals("lih")) execLih();
      if ( mAnimate )
      {
        switch( mStep ) {
        case 1 : animateRegIbus2(); break;
        case 2 :
          mArchiEngine.addAnimState(PixConstants.BUS2TOUALANIM );
          mArchiEngine.addAnimState(PixConstants.UALTOBUS3ANIM );
          mArchiEngine.addAnimState(PixConstants.BUS3ANIM );
          break;
        case 3 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    case 2 :
      if( mInstruction.getComd().equals("b")) execB(); else
      if( mInstruction.getComd().equals("ba"))execBa();
      
      if( mAnimate )
      {
        switch( mStep ) {
        case 1 : animateRegIbus2(); break;
        case 2 : animatePCBus1(); break;
        case 3 : animateUAL(); break;
        case 4 : mArchiEngine.addAnimState(PixConstants.BUS3TOCPTANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    case 3 :
      if( mInstruction.getComd().equals("beq")) execBeq(); else
      if( mInstruction.getComd().equals("bne")) execBne(); else
      if( mInstruction.getComd().equals("bge")) execBge(); else
      if( mInstruction.getComd().equals("bgt")) execBgt(); else
      if( mInstruction.getComd().equals("ble")) execBle(); else
      if( mInstruction.getComd().equals("blt")) execBlt(); else
      if( mInstruction.getComd().equals("bvs")) execBvs(); else
      if( mInstruction.getComd().equals("bvc")) execBvc();
      
      if ( mAnimate )
      {
        switch ( mStep )
        {
        case 1 : animateRegIbus2(); break;
        case 2 : animatePCBus1(); break;
        case 3 : animateUAL(); break;
        case 4 : mArchiEngine.addAnimState(PixConstants.BUS3TOCPTANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    case 4 :
      if( mInstruction.getComd().equals("add"))   execAdd(); else
      if( mInstruction.getComd().equals("add."))  execAddPt(); else
      if( mInstruction.getComd().equals("sub"))   execSub(); else
      if( mInstruction.getComd().equals("sub."))  execSubPt(); else
      if( mInstruction.getComd().equals("and."))  execAndPt(); else
      if( mInstruction.getComd().equals("or"))    execOr(); else
      if( mInstruction.getComd().equals("ldx"))   execLdx(); else
      if( mInstruction.getComd().equals("stx"))   execStx(); else
      if( mInstruction.getComd().equals("ldbx"))  execLdbx(); else
      if( mInstruction.getComd().equals("ldsbx")) execLdsbx(); else
      if( mInstruction.getComd().equals("stbx"))  execStbx();

      if( mInstruction.getComd().substring(0,2).equals("ld") && mAnimate ) {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateReg2toBus2(); break;
        case 3 : animateUAL(); break;
        case 4 : animateBus3RadmBus1(); break;
        case 5 : animateBus1MemBus3(); break;
        case 6 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      else if( mInstruction.getComd().substring(0,2).equals("st") && mAnimate)
      {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateReg2toBus2(); break;
        case 3 : animateUAL(); break;
        case 4 : animateBus3RadmBus1(); break;
        case 5 : animateReg2toBus2(); break;
        case 6 : animateBus1n2Mem(); break;
        }
        mArchiEngine.startAnim();
      }
      else if( mAnimate )
      {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateReg2toBus2(); break;
        case 3 : animateUAL(); break;
        case 4 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    case 5 :
      if( mInstruction.getComd().equals("addi")) execAddi(); else
      if( mInstruction.getComd().equals("addi.")) execAddiPt(); else
      if( mInstruction.getComd().equals("andi.")) execAndiPt(); else
      if( mInstruction.getComd().equals("ori")) execOri(); else
      if( mInstruction.getComd().equals("ld")) execLd(); else
      if( mInstruction.getComd().equals("st")) execSt(); else
      if( mInstruction.getComd().equals("ldb")) execLdb(); else
      if( mInstruction.getComd().equals("ldsb")) execLdsb(); else
      if( mInstruction.getComd().equals("stb")) execStb();

      if( mInstruction.getComd().substring(0,2).equals("ld") && mAnimate ) {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateRegIbus2(); break;
        case 3 : animateUAL(); break;
        case 4 : animateBus3RadmBus1(); break;
        case 5 : animateBus1MemBus3(); break;
        case 6 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      else if( mInstruction.getComd().substring(0,2).equals("st") && mAnimate )
      {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateRegIbus2(); break;
        case 3 : animateUAL(); break;
        case 4 : animateBus3RadmBus1(); break;
        case 5 : animateReg2toBus2(); break;
        case 6 : animateBus1n2Mem(); break;
        }
        mArchiEngine.startAnim();
      }
      else if( mAnimate )
      {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateRegIbus2(); break;
        case 3 : animateUAL(); break;
        case 4 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    case 6 :
      if( mInstruction.getComd().equals("lsl")) execLsl(); else
      if( mInstruction.getComd().equals("rol")) execRol(); else
      if( mInstruction.getComd().equals("lsr")) execLsr(); else
      if( mInstruction.getComd().equals("asr")) execAsr();
      
      if( mAnimate )
      {
        switch( mStep ) {
        case 1 : animateReg1toBus1(); break;
        case 2 : animateReg2toBus2(); break;
        case 3 : animateUAL(); break;
        case 4 : mArchiEngine.addAnimState(PixConstants.BUS3TOREGANIM ); break;
        }
        mArchiEngine.startAnim();
      }
      break;
    default : break;
    }
  }
  
  /** anime l'UAL */
  private void animateUAL()
  {
    mArchiEngine.addAnimState(PixConstants.BUS1TOUALANIM );
    mArchiEngine.addAnimState(PixConstants.BUS2TOUALANIM );
    mArchiEngine.addAnimState(PixConstants.UALTOBUS3ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS3ANIM );
  }
  
  /** anime le bus 1 */
  private void animateReg1toBus1()
  {
    mArchiEngine.addAnimState(PixConstants.REGTOBUS1ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS1ANIM );
  }
  
  /** anime le bus 2 */
  private void animateReg2toBus2()
  {
    mArchiEngine.addAnimState(PixConstants.REGTOBUS2ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS2ANIM );
  }
  
  /** anime le bus3, radm, bus1 */
  private void animateBus3RadmBus1()
  {
    mArchiEngine.addAnimState(PixConstants.BUS3TORADMANIM );
    mArchiEngine.addAnimState(PixConstants.RADMTOBUS1ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS1ANIM );
  }
  
  /** anime le Registre d'instruction, bus2 */
  private void animateRegIbus2()
  {
    mArchiEngine.addAnimState(PixConstants.REGITOBUS2ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS2ANIM );
  }

  /** anime le Bus1, Bus2, mem */
  private void animateBus1n2Mem()
  {
    mArchiEngine.addAnimState(PixConstants.BUS1TOMEMANIM );
    mArchiEngine.addAnimState(PixConstants.BUS2TOMEMANIM );
  }
  
  /** anime le PC, bus1 */
  private void animatePCBus1()
  {
    mArchiEngine.addAnimState(PixConstants.CPTTOBUS1ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS1ANIM );
  }
  
  /** anime le bus1, mem, bus3 */
  private void animateBus1MemBus3()
  {
    mArchiEngine.addAnimState(PixConstants.BUS1TOMEMANIM );
    mArchiEngine.addAnimState(PixConstants.MEMTOBUS3ANIM );
    mArchiEngine.addAnimState(PixConstants.BUS3ANIM );
  }
  
  /** Execute l'instruction correspondant a son nom */
  private void execLih()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setUalInfo("\u2205"+"2" );
      break;
    case 3 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg1()+" crit" );
      mRegEngine.setRegFromModelAt(mInstruction.getArg2()*32,mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execB()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setUalInfo("+" );
      break;
    case 4 :
      mPositionTmp = mPosition+(mInstruction.getArg1())*2/2;
      mArchiEngine.setPC(mPosition+1+(mInstruction.getArg1())*2/2 );
      mJump = true;
      break;
    default : break;
    }

  }

  /** Execute l'instruction correspondant a son nom */
  private void execBa()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setUalInfo("\u2205"+"2" );
      break;
    case 4 :
      mPositionTmp=(mInstruction.getArg1())*2/2-1;
      mArchiEngine.setPC((mInstruction.getArg1())*2/2 );
      mJump=true;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBeq()
  {
    switch( mStep ) {
    case 1 :
      if (mRegEngine.getRccZ()==1) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if (mRegEngine.getRccZ()==1)
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBne()
  {
    switch( mStep ) {
    case 1 :
      if (mRegEngine.getRccZ()==0) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if (mRegEngine.getRccZ()==0)
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBge()
  {
    switch( mStep ) {
    case 1 :
      if( (mRegEngine.getRccZ()==1) || (mRegEngine.getRccN()==0) ) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if  ((mRegEngine.getRccZ()==1)||(mRegEngine.getRccN()==0))
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBgt()
  {
    switch( mStep ) {
    case 1 :
    {
      if ((mRegEngine.getRccZ()==0)&&(mRegEngine.getRccN()==0)) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    }
    case 4 :
      if  ((mRegEngine.getRccZ()==0)&&(mRegEngine.getRccN()==0))
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBle()
  {
    switch( mStep ) {
    case 1 :
    {
      if ((mRegEngine.getRccZ()==1)||(mRegEngine.getRccN()==1)) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    }
    case 4 :
    {
      if  ((mRegEngine.getRccZ()==1)||(mRegEngine.getRccN()==1))
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    }
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBlt()
  {
    switch( mStep ) {
    case 1 :
      if ((mRegEngine.getRccZ()==0)&&(mRegEngine.getRccN()==1)) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if  ((mRegEngine.getRccZ()==0)&&(mRegEngine.getRccN()==1))
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBvs()
  {
    switch( mStep ) {
    case 1 :
      if (mRegEngine.getRccV()==1) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if  (mRegEngine.getRccV()==1)
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execBvc()
  {
    switch( mStep ) {
    case 1 :
      if (mRegEngine.getRccV()==0) mArchiEngine.setUalInfo("+" );
      else mArchiEngine.setUalInfo("\u2205"+"1" );
      break;
    case 4 :
      if  (mRegEngine.getRccV()==0)
      {
        mPositionTmp=mPosition+(mInstruction.getArg1() )*2/2;
        mArchiEngine.setPC(mPosition );
        mJump=true;
      }
      else mJump=false;
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAdd()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin(a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAddPt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt( mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt( mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      /*
        Les instructions pointees ne consomment plus le RCC
        Result res=Utils.addBin(lResultat.getResult(),"000000000000000" + mRegEngine.getRccC(),16 );
        if (lResultat.getCarry()==1) res.setCarry(1 );
      */
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execSub()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("-" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( -a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execSubPt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("-" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( -a3 ), 16 );
      /*
        Les instructions pointees ne consomment plus le RCC
        String rcC = ( (mRegEngine.getRccC()==0)? rcC = "0000000000000000" : "1111111111111111" );
        Result res=Utils.addBin( lResultat.getResult(), rcC, 16 );
        if( lResultat.getCarry() == 1 ) res.setCarry( 1 );
      */
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAndPt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("&" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.andBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execOr()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info( "Registre " + mInstruction.getArg2() + " lu" );
      mArchiEngine.setUalInfo( "|" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 4 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.orBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLdx()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem16At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( lValue, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execStx()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg1()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg2()+" lu" );
      break;
    case 5 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      int a1 = mRegEngine.getRegAt( mInstruction.getArg1() );
      int a2 = mRegEngine.getRegAt( mInstruction.getArg2() );
      int a3 = mRegEngine.getRegAt( mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a1 ), Utils.signedIntToBin( a2 ), 16 );
      int inAddr = Utils.unsignedBinToInt( lResult.getResult() );
      mRamEngine.setRamFromModel( Utils.signedIntToBin( a3 ), inAddr );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLdbx()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem8At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( lValue, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLdsbx()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 3 :
      mArchiEngine.setUalInfo("+" );
      break;
    case 6 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem8At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( ((lValue+128)%256)-128, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execStbx()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg1()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 2 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg2()+" lu" );
      break;
    case 5 :
      mArchiEngine.setReg3Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      int a1=mRegEngine.getRegAt(mInstruction.getArg1() );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a1 ), Utils.signedIntToBin( a2 ), 16 );
      int inAddr = Utils.unsignedBinToInt( lResult.getResult() );
      mRamEngine.setRamFromModel( Utils.signedIntToBin( a3 ).substring(8,16 ), inAddr );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAddi()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAddiPt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mInstruction.getArg3();
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      /*
        Les instructions pointees ne consomment plus le RCC
        Result res=Utils.addBin(lResultat.getResult(),"000000000000000"+mRegEngine.getRccC(),16 );
        if (lResultat.getCarry()==1) res.setCarry( 1 );
      */
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAndiPt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("&" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      Result lResult = Utils.andBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execOri()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("|" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt( mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      Result lResult = Utils.orBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      mRegEngine.setRegFromModelAt( Utils.signedBinToInt( lResult.getResult() ), mInstruction.getArg1() );
      mRegEngine.updtRcc( lResult );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLd()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 6 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2 = mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3 = mInstruction.getArg3();
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem16At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( lValue, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execSt()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg1()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 5 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      int a1 = mRegEngine.getRegAt(mInstruction.getArg1() );
      int a2 = mInstruction.getArg2();
      int a3 = mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a1 ), Utils.signedIntToBin( a2 ), 16 );
      int inAddr = Utils.unsignedBinToInt( lResult.getResult() );
      mRamEngine.setRamFromModel( Utils.signedIntToBin( a3 ), inAddr );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLdb()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 6 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem8At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( lValue, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLdsb()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 6 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      Result lResult = Utils.addBin( Utils.signedIntToBin( a2 ), Utils.signedIntToBin( a3 ), 16 );
      int lValue = mRamEngine.getMem8At( Utils.unsignedBinToInt( lResult.getResult() ) );
      mRegEngine.setRegFromModelAt( ((lValue+128)%256)-128, mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execStb()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg1()+" lu" );
      mArchiEngine.setUalInfo("+" );
      break;
    case 5 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg3()+" lu" );
      break;
    case 6 :
      int a1=mRegEngine.getRegAt(mInstruction.getArg1() );
      int a2=mInstruction.getArg2();
      int a3=mRegEngine.getRegAt(mInstruction.getArg3() );
      Result lResult = Utils.addBin( Utils.signedIntToBin( a1 ), Utils.signedIntToBin( a2 ), 16 );
      int inAddr = Utils.unsignedBinToInt( lResult.getResult() );
      mRamEngine.setRamFromModel( Utils.signedIntToBin( a3 ).substring( 8, 16 ), inAddr );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLsl()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("<<" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      String s2=Utils.signedIntToBin(a2 );
      String sTemp="0000000000000000";
      s2=s2.substring(a3,16)+sTemp.substring(0,a3 );
      int result=Utils.signedBinToInt(s2 );
      mRegEngine.setRegFromModelAt(result,mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execRol()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo("<<" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      String s2=Utils.signedIntToBin(a2 );
      s2=s2.substring(a3,16)+s2.substring(0,a3 );
      int result=Utils.signedBinToInt(s2 );
      mRegEngine.setRegFromModelAt(result,mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execLsr()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info("Registre "+mInstruction.getArg2()+" lu" );
      mArchiEngine.setUalInfo(">>" );
      break;
    case 4 :
      mArchiEngine.setReg2Info("Registre "+mInstruction.getArg1()+" crit" );
      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      String s2=Utils.signedIntToBin(a2 );
      String sTemp="0000000000000000";
      s2=sTemp.substring(0,a3)+s2.substring(0,16-a3 );
      int result=Utils.signedBinToInt(s2 );
      mRegEngine.setRegFromModelAt(result,mInstruction.getArg1() );
      break;
    default : break;
    }
  }

  /** Execute l'instruction correspondant a son nom */
  private void execAsr()
  {
    switch( mStep ) {
    case 1 :
      mArchiEngine.setReg1Info( "Registre " + mInstruction.getArg2() + " lu" );
      mArchiEngine.setUalInfo(">>" );
      break;
    case 4 :
      mArchiEngine.setReg2Info( "Registre " + mInstruction.getArg1() + " crit" );

      int a2=mRegEngine.getRegAt(mInstruction.getArg2() );
      int a3=mInstruction.getArg3();
      String s2=Utils.signedIntToBin(a2 );
      String sTemp;
      if (s2.substring(0,1).equals("0"))
        sTemp="0000000000000000";
      else sTemp="1111111111111111";
      s2=sTemp.substring(0,a3)+s2.substring(0,16-a3 );
      int result=Utils.signedBinToInt(s2 );
      mRegEngine.setRegFromModelAt( result, mInstruction.getArg1() );

      break;
    default : break;
    }
  }
}
