/**
* Le modele de l'architecture
*
* @version 1.0 19/03/00
* @author Paul Ladeveze
* @author Frederik Sackx
*/
package Model;

import Controlers.ArchiControl;

public class MArchi
{
  /**
   * Le controleur
   */
  private  ArchiControl mControl;

  /**
   * Le moteur (on doit le voir pour lui dire quand l'animation en cours est finie)
   */
  private Engine mEngine;

  /**
   * Le constructeur ( qui ne fait rien )
   */
  public MArchi( Engine inEngine ) { mEngine = inEngine; }
  
  /**
   * La methode qui definint le controleur
   * @param rc : le controleur de l'architecture
   */
  public void setControler(ArchiControl inControl ) { mControl = inControl; }
    
  /**
   * La methode d'acces au controleur
   * @return le controleur
   */
  public ArchiControl getControler() { return mControl; }
  
  /**
   * Met a jour le texte dans l'interface en passant par le contrleur
   */
  public void setMsg( String inStr ) { mControl.setMsg( inStr ); }
  
  /**
   * Recupere le texte de l'interface
   * @return Le texte d'info de l'interface
   */
  public String getMsg() { return mControl.getMsg(); }
  
  /**
   * Met a jour le compteur de programme
   * @param pos : la valeur du compteur de programme
   */
  public void setPC( int inPC ) { if (inPC!=0) mControl.updatePC((inPC-1) ); }
  
  /**
   * Met a jour le compteur de programme
   * @param pos : la valeur du compteur de programme
   */
  public void setUalInfo( String inStr ) { mControl.setUalInfo( inStr ); }
  
  /**
   * Met a jour le compteur de programme
   * @param pos : la valeur du compteur de programme
   */
  public void setReg1Info( String inStr ) { mControl.setReg1Info( inStr ); }
  
  /**
   * Met a jour le compteur de programme
   * @param pos : la valeur du compteur de programme
   */
  public void setReg2Info( String inStr ) { mControl.setReg2Info( inStr ); }
  
  /**
   * Met a jour le compteur de programme
   * @param pos : la valeur du compteur de programme
   */
  public void setReg3Info( String inStr ) { mControl.setReg3Info( inStr ); }
  
  /**
   * Met a jour les affichages dans l'architecture
   */
  public void updateArchi() { mControl.updateArchi(); }
  
  /**
   * Affiche l'etat de l'execution
   */
  public void afficheInstruction( int inStep , int inPos, String info )
  {
    if( inPos == 0 )
      setMsg( "Assemblage du programme " );
    else if( inStep == 0 )
      setMsg( "Lecture de l'instruction " + inPos );
    else
      setMsg( info );
  }

  /**
   * Nettoie le schema
   */
  public void clear() { mControl.clear(); }
  
  /**
   * Methode qui ajoute une animation a la liste
   * @return l'etat de l'animation
   */
  public void addAnimState( int inAnim ){ mControl.addAnimState( inAnim ); }
  
  /**
   * Methode qui demarre les animations
   */
  public void startAnim() { mControl.startAnim(); }
  
  /**
   * preaffichage des images
   */
  public void showAllpics() { mControl.showAllpics(); }
  
  /**
   * Stoppe l'animation en cours
   */
  public void stopAnim() { mControl.stopAnim(); }
  
  /**
   * Met a 0 le numero d'animation
   */
  public void resetColor() { mControl.resetColor(); }
  
  /**
   * Methode qui poursuit l'execution lorsqu'une animation est finie
   */
  public void continueExec() { mEngine.continueExec(); }
  
  /**
   * La methode qui dit si la lecture d'une instruction doit etre animee ou non
   */
  public void setAnimLect( boolean inAnimate ) { mEngine.setAnimLect( inAnimate ); }
}
