/**
 * Le modele de l'editeur
 * 
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 * @author Yves Lhuillier
 */
package Model;

import java.util.Vector;
import java.io.*;

import Controlers.EditControl;
import Controlers.UIControl;
import AsmCode.*;

public class MEdit
{
  /** La suite d'instruction assembleur sur laquelle on travaille (en chanes de carctres) */
  private  Vector mAsmContent = new Vector();
  /** La suite d'instruction assembleur sur laquelle on travaille (dans la reprsentation interne) */
  private  Vector mCodeContent = new Vector();
  /** Le controleur d'edition */
  private EditControl mEditControl;
  /** Le controleur des menus */
  private UIControl mMenuControl;
  
  /** le constructeur */
  public MEdit() {}
  
  /**
   * La methode qui definit le controleur
   * @param inEditCtrl : le controleur d'edition
   * @param inMenuCtrl : le controleur des menus
   */
  public void setControler( EditControl inEditCtrl, UIControl inMenuCtrl ) {
    mEditControl = inEditCtrl;
    mMenuControl = inMenuCtrl;
  }
  
  /**
   * La methode d'acces au controleur
   * @return le controleur d'dition
   */
  public EditControl getControler() { return mEditControl; }
  
  /**
   * La methode qui donne acces a la variable content
   * @return  le Vector content
   */
  public Vector getContent() { return mAsmContent; }
  
  /**
   * La methode qui donne acces a la variable content
   * @return  le Vector content
   */
  public Vector getProgram() { return mCodeContent; }
  
  /**
   * La methode qui change tout le mAsmContent du tableau d'instructions
   * @param content : le nouveau tableau d'instructions assembleur
   */
  public void setContent(Vector content) { mAsmContent = (Vector)content.clone(); }
  
  /**
  * La methode qui change tout le mAsmContent du tableau d'instructions a partir d'une chaine de caracteres
  * @param content : la chaine contenant les instructions assembleur
  */
  public void setContentFromString( String inContent ) { setContent( new StringReader( inContent ) ); }
  
  /**
  * La methode qui change le mAsmContent d'une case du tableau d'instructions
  * @param valeur : la nouvelle instruction
  * @param pos : l'index ou la ranger
  */
  public void setContentAt(String valeur, int pos)
  {
  while (pos>=mAsmContent.size()) {mAsmContent.addElement("" );
  }
  mAsmContent.setElementAt(valeur,pos );
  }
  
  
  /**
    * La methode qui charge le contenu d'un fichier dans le tableau d'instructions
    * @param f : le inputReader contenant des instructions assembleur
    */
  public void setContent(Reader f) {
    try {
      mAsmContent = new Vector();
      LineNumberReader file = new LineNumberReader(f );
      String lStr = "";
      while ( (lStr=file.readLine()) != null ) { mAsmContent.addElement(lStr ); }
      f.close();
      mEditControl.setUIEdit(mAsmContent );
    } catch (IOException e) { System.out.println( "Erreur lors du chargement du fichier" ); }
  }
  
  /**
    * La methode qui sauve le mAsmContent du tableau d'instructions dans un fichier
    * @param inFile : le fichier sous lequel sauver les instructions assembleur
    */
  public void sauver( File inFile ) {
    try {
      FileWriter lFile = new FileWriter( inFile );
      String lText = getText();
      lFile.write( lText );
      lFile.close();
    } catch (IOException e) { System.out.println( "Erreur lors de la sauvegarde du fichier" ); }
  }
  
  /**
    * La methode qui donne acces au mAsmContent de toutes les cases du tableau d'instructions assembleur
    * @return  le mAsmContent du tableau sous forme de chaine de caractere
    */
  public String getText() {
    String lStr="";
    for( int j = 0; j < mAsmContent.size(); j++ )
      lStr += ((String)mAsmContent.elementAt(j) + "\n" );
    return lStr;
  }
  
  /**
    * La methode qui montre la boite d'erreur
    * @param msg : le message a afficher
    */
  public void showErrorFormat( String inMsg ) { mEditControl.showErrorFormat( inMsg ); }
  
  /**
    * La methode qui montre la boite d'erreur
    * @param msg : le message a afficher
    */
  public void showErrorSyntax( String inMsg ) { mEditControl.showErrorSyntax( inMsg ); }
  
  /**
    * La methode qui montre la boite d'erreur pour une erreur de saut dans
    * le programme
    * @param msg : le message a afficher
    */
  public void showErrorJump( String inMsg ) { mEditControl.showErrorJump( inMsg ); }
  
  /**
    * La methode qui lance l'analyse lexicale et syntaxique
    * @exception Exception
    */
  public void launchAnalyse()  throws Exception { mCodeContent = mEditControl.analyse(); }
  
  /**
  * La methode qui sauve le fichier courant dans un fichier temporaire lors de l'analyse
  */
  public Vector analyse() throws Exception {
    Vector res = new Vector();
    try {
      StringReader file = new StringReader( getText() );
      res = AsmCode.parser.letsParse( file );
    } catch (IOException e) {}
    return res;
  }
  
  /**
    * Methode servant a trouver le numero de ligne d'unze instruction errone
    * @param inLine : le numero de la ligne sans tenir compte des espaces dans l'editeur
    * @return le numero de ligne en tenant compte
    */
  public int seekPosInEdit( int inLine ) { return mEditControl.seekPosInEdit( inLine ); }
  
  /**
    * Sert a rendre l'dition possible ou impossible
    * @param b : oui ou non l'dition est elle possible
    */
  public void setEditEnabled( boolean inEnabled ) {
    mEditControl.setEditEnabled( inEnabled );
    mMenuControl.setMenusEnabled( inEnabled );
  }
  
  /**
    * Stoppe toute edition
    */
  public void unEdit() { mEditControl.unEdit(); }
  
  /**
    * Selectionne la ligne de code pos dans l'editeur
    * @param inPos : la ligne de l'editeur a selectionner
    */
  public void setSelected( int inPos ) { mEditControl.setSelected( inPos ); }
  
  /** Methode qui supprime une ligne d'instruction
    * @param inPos: instruction a supprimer
    */
  public void removeLine( int inPos ) { if ( inPos < mAsmContent.size() ) mAsmContent.removeElementAt( inPos ); }
}
