/**
 * Le modele de la memoire RAM
 *
 * @version 1.0 19/03/00
 * @author Paul Ladeveze
 * @author Frederik Sackx
 */
package Model;


import java.util.*;

import Controlers.RamControl;
import Utilitaires.*;

public class MRam
{
  /**
    * Le contenu de la RAM
    */
  private  Vector mMemStorage = new Vector(65536);
  private  Vector mMemBackup;

  /**
    * Le controleur
    */
  private  RamControl mControl;

  /**
    * Une structure pour stocker les cases modifies
    */
  private Vector mAlteredAddr    = new Vector();
  private Vector mAlteredStorage = new Vector();

  /**
    * Le constructeur qui initialise le contenu de la RAM avec des valeurs aleatoires
    * @see Model.MRam#initMem()
    */
  public MRam() { initMem(); }

  /**
    * La methode qui definie le controleur
    * @param inControler : le controleur de RAM
    */
  public void setControler( RamControl inControler ) { mControl = inControler; }

  /**
    * La methode qui retourne le controleur
    */
  public RamControl getControler() { return mControl; }

  /**
    * La methode qui initialise le vector contenant la RAM
    */
  private void initMem() {
    Randomize lRand = new Randomize( 8675309 );
    for( int i = 0; i < 65536; i++ ) {
      mMemStorage.addElement( new Integer( lRand.nextInt( 13 ) - 6 ) );
    }
  }
      
  /**
    * La methode qui sauve l'etat de la memoire
    */
  public void saveRam() { mMemBackup = (Vector)mMemStorage.clone(); }
  
  /**
    * Une methode qui restaure le dernier etat de la memoire sauvegarde
    */
  public void restoreRam() {
    mMemStorage = (Vector)mMemBackup.clone();
    mControl.setRamInUI();
    mControl.refreshReg();
  }

  /**
    * La methode d'initialisation de cases modifies
    */
  public void newAlter() {
    mAlteredStorage = new Vector();
    mAlteredAddr    = new Vector();
  }


  /**
    * La methode d'acces au vector  contenant la RAM
    * @return le vecteur RAM
    */
  public  Vector getMem() { return mMemStorage; }

  /**
    * La methode qui change le contenu de la RAM a une certaine adresse
    * @param inVal: la nouveau contenu de l'adresse
    * @param inAddr: l'adresse
    */
  public void setMemAt( Integer inVal, int inAddr )
  {
    mMemStorage.setElementAt( inVal, inAddr );
    int lIndex = mAlteredAddr.indexOf( new Integer( inAddr ) );
    if ( lIndex != -1 )
      mAlteredStorage.setElementAt( inVal, lIndex );
  }

  /**
    * La methode qui accede au contenu de la RAM sur 16 bits
    * @param inAddr: l'adresse
    */
  public int getMem16At( int inAddr )
  {
    /*
    int m1 = ((Integer)mMemStorage.elementAt( inAddr )).intValue();
    int m2 = ((Integer)mMemStorage.elementAt( inAddr+1 )).intValue();
    String s1 = Utils.signedIntToBin( m1 ).substring( 8, 16 );
    String s2 = Utils.signedIntToBin( m2 ).substring( 8, 16 );
    return Utils.signedBinToInt( s1 + s2 );
    */
    return ((Integer)mMemStorage.elementAt( inAddr )).intValue();
  }

  /**
    * La methode qui accede au contenu de la RAM sur 8 bits
    * @param inAddr: l'adresse
    */
  public int getMem8At( int inAddr ) {
    /*
    return ((Integer)mMemStorage.elementAt( inAddr )).intValue();
    */
    return Utils.unsignedBinToInt(
      Utils.signedIntToBin(
        ((Integer)mMemStorage.elementAt( inAddr )).intValue()
      ).substring( 8, 16 )
    );
  }

  /**
    * La methode qui remplit le modele de la RAM a partir d'un changement dans le modele
    * lors de l'assemblage
    * @param inBin : la nouvelle valeur a mettre dans le modele et l'interface de la RAM (binary string)
    * @param inAddr : l'addresse memoire ou l'on charge inBin
    */
  public void setRamFromModel( String inBin, int inAddr ) {
    Integer  lVal    = new Integer ( Utils.signedBinToInt( inBin ) );
    int      lIndex  = mAlteredAddr.indexOf( new Integer( inAddr ) );
    
    if ( lIndex != -1 )
      mAlteredStorage.setElementAt( lVal, lIndex );
    else {
      mAlteredAddr.addElement( new Integer( inAddr ) );
      mAlteredStorage.addElement( lVal );
    }
    mControl.setRamFromModel( inBin, inAddr );
  }

  /**
    * La methode d'acces au n des cases modifiees
    * @return la RAM
    */
  public  Vector getAlteredAddr() { return mAlteredAddr; }

  /**
    * La methode d'acces au contenu des cases modifiees
    * @return la RAM
    */
  public  Vector getAlteredStorage() { return mAlteredStorage; }

  /**
    * Initialise les cases modifies
    */
  public void initAlter() { mControl.initAlter(); }
}
