/**
* Le modele des registres
*
* @version 1.0 19/03/00
* @author Paul Ladeveze
* @author Frederik Sackx
*/
package Model;


import java.util.*;

import Controlers.RegControl;
import Utilitaires.*;

public class MReg
{
  /**
   * Le contenu des registres
   */
  private  Vector mRegStorage = new Vector( 8 );
  private  Vector mRegBackup;

  /**
   * Le contenu du registre RCC
   */
  private Vector mRccStorage = new Vector( 4 );
  private Vector mRccBackup;

  /**
   * Le controleur
   */
  private  RegControl mControl;

  /**
   * Le constructeur qui initialise le contenu des registres avec des nombres aleatoires
   * sauf pour R0 qui est cabl a 0
   */
  public MReg() { initReg(); }

  /**
   * La methode qui definint le controleur
   * @param rc : le controleur des registres
   */
  public void setControler( RegControl inRegCtrl ) { mControl = inRegCtrl; }


  /**
   * La methode qui sauve l'etat des registres
   * @param rc : le controleur des registres
   */
  public void saveReg()
  {
    mRegBackup=(Vector)mRegStorage.clone();
    mRccBackup=(Vector)mRccStorage.clone();
  }


  /**
   * Une methode qui restaure le dernier etat des registres sauvegarde
   * @param rc : le controleur des registres
   */
  public void restoreReg()
  {
    mRegStorage = (Vector)mRegBackup.clone();
    mRccStorage = (Vector)mRccBackup.clone();
    mControl.setRegInUI();
    mControl.setRccInUI();
    mControl.refreshReg();
  }

  /**
   * La methode d'acces au controleur
   * @return Le controleur
   */
  public RegControl getControler() { return mControl; }

  /**
   * La methode qui initialise le vector contenant la RAM
   */
  private void initReg()
  {
    Randomize lRand = new Randomize( 8675309 );
    mRegStorage.addElement( new Integer( 0 ) );
    
    for( int i = 1; i < 8; i++ )
      mRegStorage.addElement(new Integer( lRand.nextInt( 29 ) - 14 ) );

    mRccStorage.addElement( new Integer( 0 ) );
    mRccStorage.addElement( new Integer( 0 ) );
    mRccStorage.addElement( new Integer( 0 ) );
    mRccStorage.addElement( new Integer( 0 ) );
  }

  /**
   * La methode d'acces au vector des registres
   * @return le vector REG
   */
  public  Vector getReg() { return mRegStorage; }

  /**
   * La methode d'acces a une case vector des registres
   * @param inRegIdx : la case
   * @return la valeur contenue
   */
  public  int getRegAt( int inRegIdx ) { return (((Integer)mRegStorage.elementAt(inRegIdx)).intValue() ); }

  /**
   * La methode d'acces au  registre RCC
   * @return le vector RCC
   */
  public  Vector getRcc() { return mRccStorage; }

  /**
   * calcule les valeurs a mettre dans le registre RCC
   * @param result : le resultat d'une operation qui positionne le RCC
   * @param resI : le resultat entier de l'opration
   */
  public void updtRcc( Result inResult )
  {
    // Mise a jour du registre C
    setRccFromModelAt( inResult.getCarry(), 0 );
    // Mise a jour du registre Z
    setRccFromModelAt( (inResult.getIntResult() == 0)?1:0, 1 );
    // Mise a jour du registre N
    setRccFromModelAt( (inResult.getResult().substring( 0, 1 ).equals( "1" ))?1:0, 2 );
    // Mise a jour du registre V
    setRccFromModelAt( inResult.getOverFlow(), 3 );
  }


  /**
   * La methode qui change le contenu des registres avec un numero de registre et une valeur
   * @param val: la nouveau contenu du registre
   * @param col: le numero du registre
   */
  public void setRegAt( Integer val, int col ) { mRegStorage.setElementAt( val, col ); }

  /**
   * La methode qui change le contenu des registres avec un numero de registre et une valeur
   * @param val: la nouveau contenu du registre
   * @param col: le numero du registre
   */
  public void setRegFromModelAt( int val,int col )
  {
    if (col!=0)
    {
      setRegAt( new Integer(val), col );
      mControl.setRegInUIAt( new Integer(val), col );
    }
  }

  /**
   * La methode qui change le contenu du registre RCC pour le flag i
   * @param val : la nouvelle valeur du flag i
   * @param i : le numero du flag 0:C 1:Z 2:N 3:V
   */
  public void setRccFromModelAt( int val, int i )
  {
    mRccStorage.setElementAt( new Integer(val),i );
    mControl.setRccInUIAt( val,i );
  }

  /**
   * La methode d'acces au flag C de RCC
   * @return la valeur du flag
   */
  public int  getRccC() { return ((Integer)mRccStorage.elementAt(0)).intValue(); }

  /**
   * La methode d'acces au flag Z de RCC
   * @return la valeur du flag
   */
  public int  getRccZ() { return ((Integer)mRccStorage.elementAt(1)).intValue(); }

  /**
   * La methode d'acces au flag N de RCC
   * @return la valeur du flag
   */
  public int  getRccN() { return ((Integer)mRccStorage.elementAt(2)).intValue(); }

  /**
   * La methode d'acces au flag V de RCC
   * @return la valeur du flag
   */
  public int  getRccV() { return ((Integer)mRccStorage.elementAt(3)).intValue(); }
}
